<?php

/**
 * класс работы с логом
 *
 * храним логи в json для выборки
 */
class Log
{
    /**
     * путь к файлу лога
     * @var string
     */
    private $logFile = '';

    const MAX_LOG_SIZE = 524288; //512*1024;

    /**
     * @param $_logFile "путь к файлу лога"
     */
    public function __construct($_logFile) {
        $this->logFile = $_logFile;
        $this->checkFile();
    }

    /**
     * извлечь все логи
     *
     * @param bool $clean Если Истина, то очищает лог
     * @return array
     */
    public function getAll($clean) {
        $content = ',';
        if($this->checkFile()) {
            try {
                $content = file_get_contents($this->logFile);
                if ( ($content === false) ) $content = ',';
                if ($clean) {
                    unlink($this->logFile);
                    touch($this->logFile);
                }
            }
            catch(Exception $e) { }
        }

        return Array(
            'last_date' => time(),
            'items'     => json_decode(substr_replace($content, '[', 0, 1) . ']'),
        );
    }

    /**
     * добавить запись в лог
     *
     * @param $type
     * @param $file
     * @param $line
     * @param $message
     * @return bool
     */
    public function set($type, $file, $line, $message) {

        if ( file_exists($this->logFile) && (filesize($this->logFile) > self::MAX_LOG_SIZE) ) {
            unlink($this->logFile);
            touch($this->logFile);
        }

        $item = array(
            'date' => time(),
            'type' => $type,
            'file' => $file,
            'line' => $line,
            'message' => $message
        );
        $json = ',' . json_encode($item);
        file_put_contents($this->logFile, $json, FILE_APPEND);
    }

    /**
     * проверяем файл лога на наличие
     * если его нету, создаем
     * @return bool
     */
    protected function checkFile() {
        $expl = explode('/', $this->logFile);
        $fName = $expl[count($expl)-1];
        $dir = substr($this->logFile, 0, (strlen($this->logFile) - strlen($fName)));
        if(!is_dir($dir)) {
            mkdir($dir);
        }
        if(!file_exists($this->logFile)) {
            touch($this->logFile);
        }

        return file_exists($this->logFile);
    }
}
