<?php

/**
  Утилитарный класс для работы с конфигурационным файлом. Этот конфигурационный файл сохраняет
  настройки даже после обновления бриджа.
 */
class Config {

    const CONFIG_FILE_NAME = "config.ini";

    private $fileContent = null;
    private $paramsArray = array();

    private function __construct() {
        if (!file_exists(self::CONFIG_FILE_NAME)) return;

        $this->fileContent = file_get_contents(self::CONFIG_FILE_NAME);
        $linesArray = explode("\n", $this->fileContent);
        foreach ($linesArray as $line) {
            $line = trim($line);
            // Skip comments
            if (substr($line, 0, 1 ) === '#') continue;
            //
            $paramAndValue = explode('=', $line, 2);
            if (count($paramAndValue) == 2) {
                $key = trim($paramAndValue[0]);
                $this->paramsArray[$key] = rtrim($paramAndValue[1], ';');
            }
        }
    }

    public static function getInstance() {
        static $instance = null;
        if ($instance === null) {
            $instance = new Config();
        }
        return $instance;
    }

    public function getParameter($parameterName, $defaultValue = null) {
        if (!isset($this->fileContent)) return $defaultValue;
        return isset($this->paramsArray[$parameterName]) ? $this->paramsArray[$parameterName] : $defaultValue;
    }

    public function getParamBool($parameterName, $defaultValue = false) {
        return (bool) $this->getParameter($parameterName, $defaultValue);
    }
	
	public function getParamInt($parameterName, $defaultValue = 0) {
        return (int) $this->getParameter($parameterName, $defaultValue);
    }

}
