<?php


/*Abstract DEVICE class*/

// Коды возврата
DEFINE ('EC_DEVICE_ERROR', 'DEVICE_ERROR'); //Ошибка в устройстве. Например, потеряна связь. Надо устранить ошибку и повторить орерацию
DEFINE ('EC_API_ERROR', 'API_ERROR');       //Ошибка использования API. Так делать нельзя

require_once __DIR__ . "/Log.php";//подключаем класс работы с логом

class Device {

	protected $_connected = false;

	protected $_nativeDevice = NULL;

	protected $_baudValues = array(1=>'110', 2=>'300',3=>'600',4=>'1200',5=>'2400',6=>'4800',7=>'9600',8=>'14400',9=>'19200',10=>'38400',11=>'56000',12=>'57600',13=>'115200');

	/**
	* Тип подключения к устройству
	*/
	protected $_deviceConnectionType = 0; // 0 - COM-порт

	/**
	* Номер ком-порта, на котором сидит устройство
	*/
	protected $_deviceCOMPortNumber = -1;

	/**
	* Скорость подключения к ком-порту, на котором сидит устройство
	*/
	protected $_deviceCOMPortBaud = -1;

	/**
	* Модель устройства
	*/
	protected $_deviceModel= -1;

        /**
         * IP адрес устройства, если оно подключено по сети
         */
        protected $_deviceIpAddress = -1;

        /**
         * TCP порт устройства, если оно подключено по сети
         */
        protected $_deviceTcpPort = -1;

        /**
	Уровень логгирования. 0 - no debug, 1 - error only, 2 - error and warnings, 3 - error, warning and info
	Уровень можно поменять, добавив к запросу параметр [loglevel=]
	*/
	public $debug_level = 0;

	/**
	Путь к лог-файлу
	*/
	public $logFilePath = 'logs/';

	/**
	TRUE = Автоматически подключаться к устройству. Иначе надо в явном виде вызывать Open() и Close()
	*/
	public $AutoOpen = true;


	/**
	 * Массив действий, которые умеет выполнять данное устройство
	 *
	 */
	private $_actions = array('foo'=>array('bar'));

	protected function _initialize() {
		return false;
	}

	protected function &get_NativeDevice() {
		// FIXME: этот код требует рефакторинга. Все можно сделать куда проще и читабельнее

		// если устройство не инициализировано, создаем его
		if (!isset($this->_nativeDevice) ) {
			$this->_initialize();
		}

		// если порт не открыт, открыть порт
		if (isset($this->_nativeDevice) && ($this->AutoOpen) && (! $this->_internalIsOpen($this->_nativeDevice)) ) {
			if ($this->_internalOpen($this->_nativeDevice)) {
				$this->_connected = true;
			} else {
				/*throw new Exception('Невозможно установить соединение с устройством. Ошибка: ' .
					$this->_internalGetLastError($this->_nativeDevice) . ' ' .
					$this->_internalGetLastErrorMessage($this->_nativeDevice) );
				*/
			}
		}

		return $this->_nativeDevice;
	}

	function __construct($loglevel = 0){
		if (!file_exists($this->logFilePath)) {
			if (! mkdir($this->logFilePath, 0777) ) {
				error_log('Не удалось создать папку для логов ' . $this->logFilePath);
				$this->debug_level = 0;
			}
		}
		$this->debug_level = $loglevel;
	}

	function __destruct() {
		if (isset($this->_nativeDevice) ) {
			if ( ($this->AutoOpen) && ($this->_internalIsOpen($this->_nativeDevice)) ) $this->_internalClose($this->_nativeDevice);
			unset($this->_nativeDevice);
		}
	}


	/**
	 * Проверяет текущее состояние устройства. Если на устройстве произошла ошибка, то возвращает
	 * массив с описанием ошибки. Если с устройством все ОК, то возвращает FALSE
	 * Метод используется после обращения к устройству следующим образом:
	 *   if ($errResult = $this->_isDeviceFailed(__METHOD__)) return $errResult;
	 *
	 * @param
	 * @return mixed
	 */
	protected function _isDeviceFailed($methodName, $dev = NULL){
		$device = isset($dev) ? $dev : $this->get_NativeDevice();
		$lastErrorNum = $this->_internalGetLastError($device);
		$lastErrorDesc = $this->_internalGetLastErrorMessage($device);
		if ($lastErrorNum != 0) {
			$this->log($methodName . " ----- end. ERROR. Произошла ошибка устройства: $lastErrorNum, $lastErrorDesc\n", 1);
			return array(
				'success'=>false,
				'error'=>"Произошла ошибка устройства при вызове функции [$methodName]: $lastErrorNum, $lastErrorDesc\n",
				'errcode' => EC_DEVICE_ERROR,
				'device_errnum' => $lastErrorNum,
				'device_errdesc' => $lastErrorDesc
			);
		} else {
			return false;
		}
	}

	/**
	 * Возвращает список действий, которые умеет выполнять данное устройство
	 *
	 * @param
	 * @return array
	 */
	protected function _getActionList(){
		return $this->_actions;
	}

	protected function _internalOpen($dev = NULL) {
		return false;
	}

	protected function _internalClose($dev = NULL) {
		return false;
	}

	protected function _internalIsOpen($dev = NULL) {
		return $this->_connected;
	}

	protected function _internalGetLastError($dev = NULL) {
		return 0;
	}

	protected function _internalGetLastErrorMessage($dev = NULL) {
		return 'no data';
	}


	/**
	 * Выполняет указанное действие на устройстве
	 *
	 * @param string $action Имя действия
	 * @param array $param Параметры для вызываемого действия
	 * @return
	 */
	protected function _doAction($action, $params) {
		$this->log(__METHOD__ .": Start \n", 3);
		if (!array_key_exists($action, $this->_getActionList())) {
			$this->log(__METHOD__ . ' ERROR В классе [' . get_class($this) . '] нет действия [' . $action . "]\n");
			return array ('success'=>false, 'error'=>'В классе [' . get_class($this) . '] нет действия [' . $action . ']' );
		}
		if (!is_callable (array($this, "_do_$action"))) {
			$this->log(__METHOD__ . ' ERROR В классе [' . get_class($this) . '] не реализовано действие [_do_' . $action . "]\n");
			return array ('success'=>false, 'error'=>'В классе [' . get_class($this) . '] не реализовано действие [_do_' . $action . ']' );
		}
		$this->log(__METHOD__ . " call [_do_$action] method\n", 3);
		return call_user_func(array($this, "_do_$action"), $params);
	}



	/**
	* Устанавливает тип подключения к устройству
	*   0 - COM-порт
	*/
	public function SetConnectionType ($contype) {
		$this->_deviceConnectionType = $contype;
	}

	/**
	* Возвращает nип подключения к устройству
	*   0 - COM-порт
	*/
	public function GetConnectionType() {
		return $this->_deviceConnectionType;
	}


	/**
	* Устанавливает номер ком-порта, на котором сидит устройство
	*/
	public function SetComPort ($comnumber) {
		$this->_deviceCOMPortNumber = $comnumber;
	}

	/**
	* Возвращает номер ком-порта, на котором сидит устройство
	*/
	public function GetComPort () {
		return $this->_deviceCOMPortNumber;
	}

	/**
	* Устанавливает скорость подключения к ком-порту, на котором сидит устройство
	//Скорости: 1-110, 2-300,3-600,4-1200,5-2400,6-4800,7-9600,8-14400,9-19200,10-38400,11-56000,12-57600,13-115200
	*/
	public function SetComPortBaud($baud) {
		$this->_deviceCOMPortBaud = $baud;
	}

	/**
	* Возвращает скорость подключения к ком-порту, на котором сидит устройство
	*/
	public function GetComPortBaud () {
		return $this->_deviceCOMPortBaud;
	}

	/**
	* Устанавливает модель устройства
	*/
	public function SetModel($model) {
		$this->_deviceModel= $model;
	}

	/**
	* Возвращает установленную модель устройства
	*/
	public function GetModel () {
		return $this->_deviceModel;
	}

        /**
         * Устанавливает IP адрес устройства
         */
        public function SetIpAddress($address) {
            $this->_deviceIpAddress = $address;
        }

        /**
         * Возвращает IP адрес устройства
         */
        public function GetIpAddress() {
            return $this->_deviceIpAddress;
        }

        /**
         * Устанавливает TCP порт устройства
         */
        public function SetTcpPort($port) {
            $this->_deviceTcpPort = $port;
        }

        /**
         * Возвращает TCP порт устройства
         */
        public function GetTcpPort() {
            return $this->_deviceTcpPort;
        }

        /**
	 * Открывает устройство для работы
	 *
	 * @param
	 * @return bool
	 */
	public function Open() {
		return array('success' => true);
	}

	/**
	 * Закрывает устройство для работы
	 *
	 * @param
	 * @return bool
	 */
	public function Close() {
		return array('success' => true);
	}

	/**
	 * Возвращает истину если устройство открыто
	 *
	 * @param
	 * @return
	 */
	public function IsOpen() {
		return false;
	}

	/**
	 * Возвращает код последней ошибки
	 *
	 * @param
	 * @return int
	 */
	public function GetLastError() {
		$answer = array('success' => false);

		$result = $this->_internalGetLastError();
		if ($result === false) return $answer;
		$answer['success'] = true;
		$answer['error'] = $result;

		$result = $this->_internalGetLastErrorMessage();
		if ($result !== false) $answer['message'] = $result;

		return $answer;
	}

	/**
	 * Добавляет строку в лог
	 *
	 * @param string $message Текст сообщения
	 * @param int    $level Уровень сообщения: 1 - error, 2 - warning, 3 - info
	 * @return
	 */
	public function log($message, $level = 1) {
		if ($level <= $this->debug_level ) {
			$errtype = 3;
			$errfile = 'device';
			$errline = '';
			$errstr  =  $message;

			$log = new Log(__DIR__.'./../logs/log.json');
			$log->set($errtype, $errfile, $errline, $errstr);

			if( ! ini_get('date.timezone') )
			{
			   date_default_timezone_set('GMT');
			}
			if (!error_log(date('d.m.Y H:i:s') . ' ' . $message, 3, ROOT_DIR .'/'. $this->logFilePath . date('Ymd') . '_hwbridge.log')) {
				error_log('Error apending line to log: ' . $this->logFilePath . date('Ymd') . '_hwbridge.log');
			}
		}
	}


	/**
	 * Возвращает версию драйвера
	 *
	 * @param
	 * @return
	 */
	public function GetVersion() {
		return 0;
	}


	/**
	 * Выводит описание объекта
	 *
	 * @param
	 * @return
	 */
	public function Describe() {
		echo '<h1>' . get_class($this) . '</h1><table width="100%" border="1">';
		$class = new ReflectionClass(get_class($this));
		$methods = $class->getMethods(ReflectionMethod::IS_PUBLIC);
		echo '<tr><td colspan="2"><b>Методы</b></td></tr>';
		foreach ($methods as $method){
			//var_dump($method);
			echo "<tr><td>{$method->name}</td><td>{$method->getDocComment()}</td></tr>";
		}

		$properties = $class->getProperties(ReflectionMethod::IS_PUBLIC);
		echo '<tr><td colspan="2"><b>Свойства</b></td></tr>';
		foreach ($properties as $property){
			//var_dump($method);
			echo "<tr><td>{$property->getName()}</td><td>{$property->getDocComment()}</td></tr>";
		}
		echo '</table><hr>';
		return (array('success'=>true));
	}
}
