<?php

/**

Базовый модуль для работы с дисплеями покупателя

Для просмотра всех функций модуля используйте запрос типа:
  http://api.server.com/api.php?a=display:shtrihm:Describe


Для вывода строки на экран дисплея используется функция "Print2".
  function Print2 (json array $p)

  Параметр $p['line1'] (обязательный)
	В массиве $p обязательно должен быть элемент "line1", в котором передается строка для вывода. Если текст не умещается в одну
	строку дисплея, то он автоматически переносится на вторую строку.

  Параметр $p['line2'] (необязательный)
	Если кроме line1 передан параметр line2, то текст из line1 выводится строго в первой строке дисплея, а текст из line2 - строго во второй строке дисплея.
	
  В line1 или line2 можно использовать символ "|". В этом случае, текст, расположенный правее этого символа будет прижат к правой строне.
  

Пример запроса:
  http://api.server.com/api.php?a=display:shtrihm:Print2&p={"line1":"Аскорбинка|30.00","line2":"Всего|150.00"}


*/

require 'device.php';

// Коды возврата, определенные в DEVICE.PHP
//DEFINE ('EC_DEVICE_ERROR', 'DEVICE_ERROR'); //Ошибка в устройстве. Например, потеряна связь. Надо устранить ошибку и повторить операцию

// Коды возврата для дисплея покупателя

/* Абстрактный класс для работы с дисплеем покупателя */
class Display extends Device {

	/**
	 * Максимальная длина строки в символах
	 * 
	 * @param 
	 * @return 
	 */
	public $XSize = -1;
	
	/**
	 * Количество строк в дисплее
	 * 
	 * @param 
	 * @return 
	 */
	public $YSize = -1;

	/**
	 * Список умений данного устройства
	 * 
	 */
	private $_actions = array(
			'init'      => array(), /*Инициализирует дисплей. Обычно  это очистка экрана, сброс некоторых настроек*/
			'clear'     => array(), /*Очистка экрана дисплея */
			'print'     => array('line1'), /*Выводит на дисплей строку line1. Если передан параметр line2, то line1 выводитcя только на первой строке, а line2 на второй*/
			'sendline'  => array('string'), /*Прямой ввод строки символов. Может использоваться для прямого ввода команд протокола дисплея. */
		);
		

	/**
	 * Возвращает список действий, которые умеет выполнять данное устройство
	 * 
	 * @param
	 * @return array
	 */
	protected function _getActionList(){
		return $this->_actions;
	}
	
	
	/************************************************/
	/* DISPLAY ACTIONS METHODS */

	
	/**
	 * Инициализирует дисплей
	 * 
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_init($params){
		return true;
	}
	
	/**
	 * Очищает дисплей
	 * 
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_clear($params){
		return true;
	}
	
	/**
	 * Выводит на дисплей строку
	 * 
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_print($params){
		return true;
	}

	/**
	 * Прямой ввод строки символов. Может использоваться для прямого ввода команд протокола дисплея
	 * 
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_sendline($params){
		return true;
	}

	/**
	 * Инициализация дисплея. Действия  с  дисплеем – см.  в  документации  производителя.  Обычно это очистка экрана, сброс некоторых настроек
	 * 
	 * @param 
	 * @return 
	 */
	public function Init($params) {
		return $this->_doAction('init', $params);
	}

	/**
	 * Очистка экрана дисплея
	 * 
	 * @param 
	 * @return 
	 */
	public function Clear($params) {
		return $this->_doAction('clear', $params);
	}

	/**
	 * Выводит  на дисплей строку.
	 *
	 * @param array $params Массив параметров. 
	 *    Если передан только line1, то он выводится на девайс как есть без каких-либо преобразований
	 *    Если кроме line1 присутствует line2, то line1 выводится на первой строке, line2 на второй
	 *    Если в line1 или line2 присутствует символ | , то все что до этого символа прижимается к левому краю, а то, что правее символа - к правому
	 * 
	 * @return 
	 */
	public function Print2($params) {
		return $this->_doAction('print', $params);
	}
	
	/**
	 * Вывод на экран сообщения о скорости обмена и номере СОМ порта для визуальной проверки наличия связи с устройством.
	 * 
	 * @param 
	 * @return 
	 */
	public function Test($params) {
		return $this->_doAction('test', $params);
	}	

	/**
	 * Возвращает количество строк и максимальную длину каждой строки, поддерживаемую устройством
	 * 
	 * @param 
	 * @return 
	 */
	public function GetSize($params) {
		return array('success'=>true, 'x' => $XSize, 'y' => $YSize);
	}	

	/**
	 * Прямой ввод строки символов. Может использоваться для прямого ввода команд протокола дисплея
	 * 
	 * @param 
	 * @return 
	 */
	public function SendLine($params) {
		return $this->_doAction('sendline', $params);
	}	

	
	
}