<?php

require 'device.php';

// Коды возврата
//DEFINE ('EC_DEVICE_ERROR', 'DEVICE_ERROR'); //Ошибка в устройстве. Например, потеряна связь. Надо устранить ошибку и повторить орерацию
//DEFINE ('EC_API_ERROR', 'API_ERROR');       //Ошибка использования API. Так делать нельзя
DEFINE('EC_NO_PAPER', 'NO_PAPER'); //Нет бумаги, надо вставить новую ленту
DEFINE('EC_CONTINUE_OR_CANCEL_CHECK', 'CONTINUE_OR_CANCEL_CHECK'); //Есть незавершенный документ, который надо либо отменить (CancelCheck), либо допечатать (ContinuePrint)
DEFINE('EC_DISCOUNT_TOO_BIG', 'DISCOUNT_TOO_BIG'); //Ошибка возникает, если размер скидки превышает сумму чека. В идеальном мире такого быть не должно.
DEFINE('EC_AMOUNT_CHECK_FAIL', 'AMOUNT_CHECK_FAIL'); //Ошибка возникает, если перед закрытием чека проверяется его сумма. Сумма получившегося чека должна точно совпадать с переданным контрольным значением
DEFINE('EC_SESSION_24HOURS_BLOCK', 'SESSION_24HOURS_BLOCK'); //Ошибка возникает, если текущая открытая смена длится более 24 часов. Для продолжения работы надо закрыть смену
DEFINE('EC_NO_OPEN_SESSION', 'NO_OPEN_SESSION'); //Ошибка возникает, если пытаются закрыть смену, а она не открыта
//Не кодировать многобайтные символы Unicode (по умолчанию они кодируются как \uXXXX). Доступна начиная с PHP 5.4.0.
if (!defined('JSON_UNESCAPED_UNICODE'))
	DEFINE('JSON_UNESCAPED_UNICODE', 0); //Fix for php 5.3


	/* Абстрактный класс для работы с фискальным регистратором/принтером чеков */

class FiscalPrinter extends Device {

	/**
	 * Список умений данного устройства
	 * Вадидатор...
	 */
	private $_actions = array(
		'print' => array('string'), /* Печатает стоку string на чеке */
		'printex' => array('string'), /* Печатает стоку string на чеке с возможностью подстановки параметров */
		'opencheck' => array(), /* Открыть чек */
		'sale' => array('item', 'price', 'quantity'), /* Отображает операцию "продажа" в чеке */
		'feed' => array('linesQuantity'), /* Вытягивает чек на linesQuantity строк */
		'discount' => array('amount'), /* Отображает в чеке скидку */
		'closecheck' => array('payment'), /* Закрывает чек и печатает ИТОГО */
		'cut' => array(), /* Обрезка чека */
		'printbarcode' => array('barcode'), /* Печатает штрихкод barcode */
		'drow' => array('linestart', 'linend'), /* Печатает графику (записанную в памяти ФР) */
		'openDrawer' => array(), /* Открыть денежный ящик */
		'returnsale' => array('item', 'price', 'quantity') /* Печать чека на возврат */
	);

	/* Список действий, регистрирующих продажу */
	private $_fiscalActions = array('sale', 'returnsale');

	/**
	 * Номер последнего чека в рамках смены
	 */
	protected $_lastCheckNumber = -1;

	/**
	 * Сквозной номер последнего чека
	 */
	protected $_lastGlobalCheckNumber = -1;

	/**
	 * Сумма сдачи
	 */
	protected $_change = -1;

	/**
	 * Сумма последнего чека
	 */
	protected $_lastCheckAmount = -1;

	/**
	 * Номер текущей смены
	 */
	protected $_sessionNumber = -1;

	/**
	 * Текс для печати в начале чека
	 */
	public $CheckTextBefore = '';

	/**
	 * Текс для печати в конце чека
	 */
	public $CheckTextAfter = '';

        /**
         * Сумма НДС в чеке 10%
         * @var type double
         */
        public $vat10Sum = 0;

        /**
         * Сумма НДС в чеке 18%
         * @var type double
         */
        public $vat18Sum = 0;

    /**
     * Сумма НДС в чеке 20%
     * @var float $vat20Sum
     */
    public $vat20Sum = 0;

        /**
         * Фискальный признак документа
         * @var type string
         */
        public $fiscalSign = "";

        /**
         * Флаг указывает, изменилось ли количество учтенных денег в аппарате. Если нет, то возвращается ошибка.
         * По умолчанию true.
         * @var type boolean
         */
        public $isMoneySumChanged = true;

        /**
	 * Возвращает список действий, которые умеет выполнять данное устройство
	 *
	 * @param
	 * @return array
	 */
	protected function _getActionList() {
		return $this->_actions;
	}

	/**
	 * Возвражает массив с действиями, которые являются регистрирующими
	 *
	 * @param
	 * @return  array
	 */
	protected function _getFiscalActions() {
		return $this->_fiscalActions;
	}

	/**
	 * Возвращает TRUE если указанное действие является регистрирующей операцией в чеке
	 *
	 * @param string $action Имя действия
	 * @return bool
	 */
	protected function _isFiscalAction($action) {
		return in_array($action, $this->_getFiscalActions());
	}

	/*	 * ********************************************* */
	/* PRINTER ACTIONS METHODS */

	/**
	 * Печатает строку на чеке
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_print($params) {
		$this->log(' - WARNING - '.__METHOD__.' Not Implemented', 1);
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Печатает строку на чеке с возможностью подстановки параметров
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_printex($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Проводит операцию продажа, печатает ее на чеке
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_sale($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Прогон чека на несколько строк вперед
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_feed($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Печатает в чеке скидку
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_discount($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Открывает чек для добавления позиций
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_opencheck($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Закрывает чек
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_closecheck($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Обрезка чека
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_cut($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Печать штрихкода ean13
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_printbarcode($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Печать изображения из загруженного имеджсета
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_drow($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Открыть денежный ящик
	 * @param type $params
	 */
	protected function _do_openDrawer($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Возвращает последний номер чека в текущей смене
	 *
	 * @param
	 * @return номер чека
	 */
	public function GetLastCheckNumber() {
		return false;
	}

	/**
	 * Возвращает общую информацию о ФР
	 *
	 * @param
	 * @return номер чека
	 */
	public function GetInfo($params) {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Возвращает номер последней закрытой смены
	 *
	 * @param
	 * @return номер чека
	 */
	public function GetLastSessionNumber() {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Возвращает success=true в том случае, если принтер готов напечатать новый чек
	 *
	 * @param
	 * @return
	 */
	public function ReadyForNewCheck() {
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}

	/**
	 * Возвращает регистрационный номер фискального регистратора
	 *
	 * @param
	 * @return
	 */
	protected function _internalGetMachineNumber($dev = NULL) {
		return null;
	}

        /**
        * Возвращает серийный номер фискального регистратора
        *
        * @param
        * @return
        */
       protected function _internalGetSerialNumber($dev = NULL) {
           return null;
       }

        /**
	 * Возвращает дату и время фискального регистратора
	 *
	 * @param
	 * @return
	 */
	protected function _internalGetDateTime($dev = NULL) {
		return null;
	}

	/**
	 * Возвращает true если есть бумага
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _checkPaper() {
		return true;
	}

	/**
	 * Метод переводит ФР в режим «Открытой смены»
	 *
	 * @param
	 * @return
	 */
	public function OpenSession() {
		return false;
	}

	/**
	 * Проверяет, готов ли принтер продолжить печатать чек после обрыва ленты
	 *
	 * @param
	 * @return bool true если принтер готов печатать дальше
	 */
	public function ReadyForContinuePrint() {
		return false;
	}

	/**
	 * Продолжает печатать чек после обрыва ленты
	 *
	 * @param
	 * @return
	 */
	public function ContinuePrint() {
		return false;
	}

	/**
	 * Проверяет поддерживается ли устройством команда "продолжить печать"
	 * @return boolean
	 */
	protected function ContinuePrintSupport() {
		return true;
	}

        protected function beforeCheckPrintArray($lines) {
        }

        /**
         * Необходимо ли конвертировать список команд для печати чека
         * @return boolean
         */
        protected function isCheckPrintArrayConvertingRequired($isLegacy) {
            return false;
        }

        protected function convertCheckPrintArray($lines) {
            return $lines;
        }

        /**
	 * Метод печатает сменный отчет
	 *
	 * @params['withcleaning'] bool - если true, то отчет с гашением
	 * @return
	 */
	public function PrintReport($params) {
		return false;
	}

	/**
	 * Получает информацию о смене
	 */
	public function GetShiftInfo() {
		return false;

		/*
		 * Результат метода: получение информации по смене
		 */
	}

	/**
	 * Печатает чек целиком из массива
	 *
	 * @param array $checkData массив вида array('meta'=>array(), 'lines'=>array())
	 * @return array
	 */
	public function CheckPrintArray($checkData) {
		$this->log(__METHOD__ . " ----- start \n", 3);
		# Выполняем проверки переданного массива
		if (!isset($checkData['meta']) || !isset($checkData['lines'])) {
			$this->log(__METHOD__ . " ----- end. Result = fail,  В массиве чека отсутствует обязательная секция [meta] или [lines]\n", 1);
			return array('success' => false, 'error' => 'В массиве чека отсутствует обязательная секция [meta] или [lines].');
		}
		$meta = $checkData['meta'];
		$lines = $checkData['lines'];
		$operationCompleted = 0;

		// если передан true в ignoreSession24Block, то не делаем проверку смены
		if (isset($meta['ignoreSession24Block']))
			$ignoreSession24Block = $meta['ignoreSession24Block'] === true ? true : false;
		else
			$ignoreSession24Block = false;

		$this->_lastCheckAmount = -1;
		$this->_lastCheckNumber = -1;
		$this->_lastGlobalCheckNumber = -1;
		$this->_change = -1;

		$ACTIONS = $this->_getActionList();

		#Проверяем строки чека на предмет ошибок
		$lineCount = 0;
		foreach ($lines as $line) {
			$lineCount++;
			if (!isset($line['action']) || !array_key_exists($line['action'], $ACTIONS)) {
				$this->log(__METHOD__ . " ----- end. Result = fail, В строке $lineCount раздела [lines] указано неизвестное действие action=" . $line['action'] . "\n", 1);
				return array('success' => false, 'error' => "В строке $lineCount раздела [lines] указано неизвестное действие action=" . $line['action']);
			}
			foreach ($ACTIONS[$line['action']] as $actionParam) {
				if (!isset($line['params'][$actionParam]) || !isset($line['params'][$actionParam])) {
					$this->log(__METHOD__ . " ----- end. Result = fail, В строке $lineCount раздела [lines] для действия action=" . $line['action'] . " не задан обязательный параметр [$actionParam]\n", 1);
					return array('success' => false, 'error' => "В строке $lineCount раздела [lines] для действия action=" . $line['action'] . " не задан обязательный параметр [$actionParam]");
				}
			}
		}

		#Продолжаем печатать чек после того, как закончилась бумага
		if (isset($meta['continue']) && ($meta['continue'])) {

			if ($this->ContinuePrintSupport()) {
				$this->log(__METHOD__ . " Получена команда на продолжение печати чека после обрыва ленты\n", 3);
				# Проверяем, готов ли принтер к печати нового чека
				$this->log(__METHOD__ . " Проверяем, готов ли принтер к продолжению печати\n", 3);
				$result = $this->ReadyForContinuePrint();
				if (!$result['success']) {
					if (isset($meta['lastactionname']))
						$result['lastactionname'] = $meta['lastactionname'];
					$this->log(__METHOD__ . " ----- end. Result = fail. Принтер не готов продолжить печать чека. result=" . json_encode($result, JSON_UNESCAPED_UNICODE) . "\n", 1);
					return $result;
				}
				$this->log(__METHOD__ . " Принтер готов. Продолжаем печать чека.\n", 3);
				$result = $this->ContinuePrint();
				if (!$result['success']) {
					if (isset($meta['lastactionname']))
						$result['lastactionname'] = $meta['lastactionname'];
					$this->log(__METHOD__ . " ----- end. Result = fail. Принтер попытался продолжить печать чека, но произошла ошибка. result=" . json_encode($result, JSON_UNESCAPED_UNICODE) . "\n", 1);
					return $result;
				}
				//Функция ContinuePrint() обязательно должна вернуть количество операций, которая она напечатала на повторном чеке
				$operationCompleted = $result['quantityOfOperations'];
			} else {
				return array(
					'success' => false,
					'error' => 'Операция продолжить печать на данном устройстве не поддерживается. Аннулируйте чек и напечатайте его снова',
					'errcode' => DEVICE_ERROR,
					'ecrmode' => $device->Mode,
					'ecradvmode' => $device->AdvancedMode
				);
			}
		} else {

			# Проверяем, готов ли принтер к печати нового чека
			$this->log(__METHOD__ . " Получена команда на печать нового чека\n", 3);
			$result = $this->ReadyForNewCheck($ignoreSession24Block);
			if (!$result['success']) {
				$this->log(__METHOD__ . " ----- end. Result = fail. Принтер не готов к печати нового чека. result=" . json_encode($result, JSON_UNESCAPED_UNICODE) . "\n", 1);
				return $result;
			}
		}

		# Выполняем команды для печати чека
		$startFromCommand = $operationCompleted;
		$this->log(__METHOD__ . " Начинается построчное выполнение команд для печати чека. Номер первой команды: $startFromCommand\n", 3);
		$lastAction = 0;

                $isLegacy = false;
                foreach ($lines as $line) {
                    if (isset($line['action']) && ($line['action'] == "sale" || $line['action'] == "returnsale")) {
                        if (!isset($line['params']['priceWithFinalDiscount'])) {
                            $isLegacy = true;
                            break;
                        }
                    }
                }
                if ($this->isCheckPrintArrayConvertingRequired($isLegacy)) {
                    $originalLines = $lines;
                    $lines = $this->convertCheckPrintArray($originalLines);
                }
                $this->beforeCheckPrintArray($lines);
		foreach ($lines as $line) {

			if ($startFromCommand <= 0) {
				$this->log(__METHOD__ . " Исполняется команда [{$line['action']}] с параметрами " . json_encode($line['params'], JSON_UNESCAPED_UNICODE) . "\n", 3);
				$result = $this->_doAction($line['action'], $line['params']);

				if (!$result['success']) {
					//Если уже была выполнена операция закрытия чека, то возвращаем "OK" даже если упали
					if ($this->_lastCheckNumber != -1 && $this->isMoneySumChanged) {
						$originalResult = $result;
						$result['success'] = true;
						$result['amount'] = $this->_lastCheckAmount;
						$result['checknum'] = $this->_lastCheckNumber;
						$result['globalchecknum'] = $this->_lastGlobalCheckNumber;
						$result['change'] = $this->_change;
						$result['session'] = $this->_sessionNumber;
						$result['serial'] = $this->_internalGetSerialNumber();
                                                $result['fiscalSign'] = $this->fiscalSign;
                                                $result['datetime'] = $this->_internalGetDateTime();
						$result['lastactionid'] = $lastAction;
						$result['lastactionname'] = $line['action'];
						$result['vat10Sum'] = $this->vat10Sum;
						$result['vat18Sum'] = $this->vat18Sum;
						$result['vat20Sum'] = $this->vat20Sum;
						$this->log(__METHOD__ . " ----- end. Result = OK+WARNING. Чек сформирован и закрыт, но команды после закрытия чека выполнены не полностью. Ошибка на выполнении команды #{$lastAction} [{$line['action']}] с параметрами " . json_encode($line['params'], JSON_UNESCAPED_UNICODE) . "; result=" . json_encode($originalResult, JSON_UNESCAPED_UNICODE) . "\n", 1);
					} else {
						$result['lastactionid'] = $lastAction;
						$result['lastactionname'] = $line['action'];
						$this->log(__METHOD__ . " ----- end. Result = fail. Ошибка на исполнении команды [{$line['action']}] с параметрами " . json_encode($line['params'], JSON_UNESCAPED_UNICODE) . "; result=" . json_encode($result, JSON_UNESCAPED_UNICODE) . "\n", 1);
					}
					return $result;
				}
			} else {
				$this->log(__METHOD__ . " Пропущена команда [{$line['action']}] с параметрами " . json_encode($line['params'], JSON_UNESCAPED_UNICODE) . "\n", 3);
				if ($this->_isFiscalAction($line['action']))
					$startFromCommand--;
			}
			$lastAction++;
		}
		$this->log(__METHOD__ . " Построчное выполнение команд для печати чека завершено\n", 3);
		$this->log(__METHOD__ . " Данные о последнем чеке: amount={$this->_lastCheckAmount}, checknum={$this->_lastCheckNumber}, globalchecknum={$this->_lastGlobalCheckNumber}, change={$this->_change}\n", 3);
		$this->log(__METHOD__ . " ----- end. Result = OK \n", 3);
		return array('success' => true,
			'amount' => $this->_lastCheckAmount,
            'vat10Sum' => $this->vat10Sum,
            'vat18Sum' => $this->vat18Sum,
            'vat20Sum' => $this->vat20Sum,
            'checknum' => $this->_lastCheckNumber,
			'globalchecknum' => $this->_lastGlobalCheckNumber,
			'change' => $this->_change,
			'session' => $this->_sessionNumber,
			'serial' => $this->_internalGetSerialNumber(),
			'fiscalSign' => $this->fiscalSign,
			'datetime' => $this->_internalGetDateTime()
		);
	}

	/**
	 * Отменяет текущий открытый чек
	 *
	 * @param
	 * @return
	 */
	public function CheckCancel() {
		return array('success' => true);
	}

	/**
	 * Возвращает серийный номер ФР
	 *
	 * @param
	 * @return
	 */
	public function GetSerialNumber() {
		$serial = $this->_internalGetSerialNumber();
		if (isset($serial)) {
			return array('success' => true, 'serial' => $serial);
		} else {
			return array('success' => false);
		}
	}

	/**
	 * Возвращает datetime фискального регистратора
	 *
	 * @param
	 * @return
	 */
	public function GetDateTime() {
		$datetime = $this->_internalGetDateTime();
		if (isset($datetime)) {
			return array('success' => true, 'datetime' => $datetime);
		} else {
			return array('success' => false);
		}
	}

	/**
	 * Метод регистрирует внесение денежной суммы в кассу.
	 *
	 * @param['amount'] - сумма внесения в рублях
	 * @return
	 */
	public function CashIncome($params) {
		return array('success' => false, 'error' => 'Method not implemented', 'errcode' => EC_API_ERROR);
	}

	/**
	 * Метод регистрирует выплату из кассы денежной суммы
	 *
	 * @param['amount'] - сумма выплаты в рублях
	 * @return
	 */
	public function CashOutcome($params) {
		return array('success' => false, 'error' => 'Method not implemented', 'errcode' => EC_API_ERROR);
	}

	/**
	 * Открыть денежный ящик
	 *
	 * @param
	 * @return
	 */
	public function OpenDrawer($params) {
		return $this->_do_openDrawer($params);
	}

	/**
	 * Напечатать штрихкод
	 *
	 * @param
	 * @return
	 */
	public function printbarcode($params) {
		return $this->_do_printbarcode($params);
	}


	public function getStringSize() {
		throw new Exception('not implemented exeption');
	}

	protected function alignString($str) {
		$XSize = $this->getStringSize();
		$postfix = '';

		if (strpos($str, '|') !== false) {
			$elements = explode('|', $str, 2);
			if ($elements) {
				$str = $elements[0];
				$postfix = substr(' ' . $elements[1], 0, $XSize);
				$XSize -= strlen($postfix);
			}
		}

		$str = substr($str, 0, $XSize);
		$str = str_pad($str, $XSize) . $postfix;
		return $str;
	}

	public function InitConfig($params)
	{
		return array('success' => false, 'error' => 'NotImplemented', 'method' => __METHOD__);
	}
}
