<?php

require 'core/display.php';
require 'display_rs232_native.php';

class display_rs232 extends Display {
	
	protected $__LastError = 0;
	protected $__LastErrorMsg = '';

	public function __construct($loglevel = 0) {
		parent::__construct($loglevel);
		$this->XSize = 20; 
		$this->YSize = 2; 
	}

	protected function _getActionList(){
		$result = parent::_getActionList();
		$result['test'] = array();
		return $result;
	}

	protected function _initialize() {
		parent::_initialize();
		$this->__LastError = 0;
		$this->__LastErrorMsg = '';
		if (!isset($this->_nativeDevice)) {
			$this->_nativeDevice = new Rs232MDisplayNative();
		}
	}
	
	protected function _internalOpen($dev = NULL) {
		$this->log(__METHOD__ . " ----- start\n", 3);
		$device = isset($dev) ? $dev : $this->get_NativeDevice();
		
		$this->log(__METHOD__ . " Connecting to device...\n", 3);
		if ($device->PortOpen) $device->PortOpen = false;
		if ($this->_deviceCOMPortNumber != -1) $device->CommPort = $this->_deviceCOMPortNumber;
		$device->Settings = '9600,n,8,1';
		if ($this->_deviceCOMPortBaud != -1) {
			if (! array_key_exists($this->_deviceCOMPortBaud, $this->_baudValues) ) {
				$this->__LastError = 1;
				$this->__LastErrorMsg = "Неверно задана скорость для COM-порта (значение: {$this->_deviceCOMPortBaud})";
				$this->log(__METHOD__ . " ----- ERROR connecting. Bad com port baud value: {$this->_deviceCOMPortBaud}\n", 3);
				return false;
			}
			$device->Settings = $this->_baudValues[$this->_deviceCOMPortBaud] . ',n,8,1';
		}
		$this->log(__METHOD__ . " Using COM-port: {$device->CommPort}:{$device->Settings} (settings: {$this->_deviceCOMPortNumber}:{$this->_deviceCOMPortBaud})...\n", 3);

		$device->Handshaking = 0; //comNone
		$device->RThreshold = 1;
		$device->InputLen = 0;
		$device->InputMode = 0; //comInputModeText

		try {
			$device->PortOpen = true;
			//Исправление кодировки
			$device->Output = chr(0x1B).chr(0x40);     //проинициализировали режим CD5220
			$device->Output = chr(0x1B).chr(0x74).chr(0x06); // установка кодовой страницы 866
			//$fix_encode = Chr(0x1B).Chr(0x3D).Chr(0x02).Chr(0x1B).Chr(0x74).Chr(0x06).Chr(0x1B).Chr(0x52).Chr(0x00);
			//$device->SendLine($fix_encode);
		} catch (Exception $e) {
			$this->log(__METHOD__ . " ----- ERROR connecting. " . $e->getMessage() . " \n", 1);
			$this->__LastError = 1;
			$this->__LastErrorMsg = $e->getMessage();
			return false;
		}
		$this->_connected = true;
		$this->log(__METHOD__ . " ----- Connected OK\n", 3);
		return true;
	}
	
	protected function _internalClose($dev = NULL) {
		$this->log(__METHOD__ . " ----- start\n", 3);
		$device = isset($dev) ? $dev : $this->get_NativeDevice();
		$this->log(__METHOD__ . " Disconnecting from device...\n", 3);
		try {
			$device->PortOpen = false;
		} catch (Exception $e) {
			$this->log(__METHOD__ . " ----- ERROR closing comm port. " . $e->getMessage() . " \n", 1);
			$this->__LastError = 1;
			$this->__LastErrorMsg = $e->getMessage();
			return false;
		}
		$this->_connected = false;
		$this->log(__METHOD__ . " Disonnected OK\n", 3);
		return true;	
	}
	
	protected function _internalGetLastError($dev = NULL) {
		return $this->__LastError;
	}	

	protected function _internalGetLastErrorMessage($dev = NULL) {
		return $this->__LastErrorMsg;
	}			
	
	
	/*OVERRIDE DEVICE METHODS*/
	
	/**
	 * Устанавливает соединение с устройством для дальнейшей работы (открывает COM порт)
	 * 
	 * @param
	 * @return bool
	 */
	public function Open() {
		parent::Open();
		$result = $this->_internalOpen();
		if (!$result) return $this->_isDeviceFailed(__METHOD__);
		return array('success' => true);
	}

	/**
	 * Закрывает соединение с устройством (закрывает COM порт)
	 * 
	 * @param
	 * @return bool
	 */
	public function Close() {
		parent::Close();
		$result = $this->_internalClose();
		if (!$result) return $this->_isDeviceFailed(__METHOD__);
		return array('success' => true);
	}	

	/**
	 * Возвращает ИСТИНА если соединение с устройством установлено
	 * 
	 * @param
	 * @return bool
	 */
	public function IsOpen() {
		$result = $this->_internalIsOpen();
		return array('success' => true, 'isopen' => $result);
	}
	
	
	
	/**
	 * Инициализирует дисплей
	 * 
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_init($params){
		$this->__LastError = 0;
		$this->__LastErrorMsg = '';
		return array('success'=>true);
	}
	
	/**
	 * Очищает дисплей
	 * 
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_clear($params){
		$this->log(__METHOD__ . " ----- start\n", 3);
		$device = $this->get_NativeDevice();
		try {
			$device->Output = chr(0x0C); // очистка
		} catch (Exception $e) {
			$this->__LastError = 1;
			$this->__LastErrorMsg = $e->getMessage();
		
			return $e->getMessage();
		}
		$this->log(__METHOD__ . " ----- end OK\n", 3);
		return array('success'=>true);
	}
	
	
	private function alignString($str){
		$XSize = $this->XSize;
		$postfix = '';
		
		if (strpos($str, '|') !== false) {
			$elements = explode('|', $str, 2);
			if ($elements) {
				$str = $elements[0];
				$postfix = substr(' ' . $elements[1], 0, $XSize);
				$XSize -= strlen($postfix);
			}
		}
	
		$str = substr($str, 0, $XSize);
		$str = str_pad($str, $XSize).$postfix;
		return $str;
	}
	
	/**
	 * Выводит на дисплей строку. 
	 * 
	 * @param array $params Массив параметров. 
	 *    Если передан только line1, то он выводится на девайс как есть без каких-либо преобразований
	 *    Если кроме line1 присутствует line2, то line1 выводится на первой строке, line2 на второй
	 *    Если в line1 или line2 присутствует символ | , то все что до этого символа прижимается к левому краю, а то, что правее символа - к правому
	 * @return bool
	 */
	protected function _do_print($params){
		$this->log(__METHOD__ . " ----- start\n", 3);
		$line1 = $params['line1']; 
		$pos = isset($params['pos']) ? $params['pos'] : 1;
		if (is_string($line1)) { 
			$line1 = iconv('UTF-8', 'CP866', $params['line1']);
			$line2 = '';
			if (isset($params['line2'])) {
				$line2 = iconv('UTF-8', 'CP866', $params['line2']);
				$line1 = $this->alignString($line1);
				$line2 = $this->alignString($line2);
			}
			$device = $this->get_NativeDevice();
			if (!$device->PortOpen) {
				return array('success' => false, 'error' => $this->_internalGetLastErrorMessage($device));
			}
			//Выводит на дисплей строку(Строка), указанную во втором параметре, с позиции, указанной в НомерПозиции
			try {
				//$device->Output = chr(0x0C); // очистка
				$device->Output = $line1;
				if (!empty($line2)) $device->Output = $line2;
			} catch (Exception $e) {
				$this->log(__METHOD__ . " ----- end Fail. " . $e->getMessage() . "\n", 1);
				$this->__LastError = 1;
				$this->__LastErrorMsg = $e->getMessage();				
				return $e->getMessage();
			}
		} else {
			$this->log(__METHOD__ . " ----- end Fail. Параметр line1 должен содержать строку\n", 1);
			$this->__LastError = 1;
			$this->__LastErrorMsg = "Параметр line1 должен содержать строку";	
			return array('success'=>false, 'error' => 'Параметр [line1] должен содержать строку', 'time' => time());
		}
		$this->log(__METHOD__ . " ----- end OK\n", 3);
		return array('success'=>true, 'time' => time());
	}
	
	/**
	 * Выводит на дисплей строку
	 * 
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_test($params){
		$this->log(__METHOD__ . " ----- start\n", 3);
		$device = $this->get_NativeDevice();
		if ($this->_internalGetLastError($device) != 0) 
			return array('success' => false, 'error' => $this->_internalGetLastErrorMessage($device));
		
		return $this->_do_print(array('line1' => 'TECT: OK', 'line2' => 'время: ' . date('H:i:s')));
	}	

	
	/***************************************
	** PUBLIC METHODS 
	*****************************************/
	
	/**
	 * Возвращает версию драйвера
	 * 
	 * @param 
	 * @return 
	 */
	public function GetVersion() {
		$this->log(__METHOD__ . " ----- start\n", 3);
		$device = $this->get_NativeDevice();
		$result = 'n/a';
		return array('success'=>true, 'version' => $result, 'time' => time());
	}	

	
	/**
	 * Возвращает размеры экрана
	 * 
	 * @param 
	 * @return 
	 */
	public function GetSize($params) {
		$this->log(__METHOD__ . " ----- start\n", 3);
		if ($errResult = $this->_isDeviceFailed(__METHOD__)) return $errResult;
		$this->log(__METHOD__ . " ----- end OK\n", 3);
		return array('success'=>true, 'x' => $this->XSize, 'y' => $this->YSize, 'time' => time());
	}

	
	
}