<?php

require 'core/display.php';
require 'display_shtrihm_native.php';

class display_shtrihm extends Display {

    public function __construct($loglevel = 0) {
        parent::__construct($loglevel);
        $this->XSize = 20;
        $this->YSize = 2;
    }

    protected function _getActionList() {
        $result = parent::_getActionList();
        $result['test'] = array();
        return $result;
    }

    protected function _initialize() {
        parent::_initialize();
        if (!isset($this->_nativeDevice)) {
            $this->_nativeDevice = new ShtrihMDisplayNative();
        }
    }

    protected function _internalOpen($dev = NULL) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = isset($dev) ? $dev : $this->get_NativeDevice();
        $this->log(__METHOD__ . " Connecting to device...\n", 3);
        if ($this->_deviceCOMPortNumber != -1)
            $device->PortNumber = $this->_deviceCOMPortNumber;
        if ($this->_deviceCOMPortBaud != -1)
            $device->BaudRate = $this->_deviceCOMPortBaud;
        if ($this->_deviceModel != -1) {
            switch ($this->_deviceModel) {
                case 1:
                    $fix_encode = Chr(0x1B) . Chr(0x74) . Chr(0x06); // установка 6 кодовой страницы
                    break;
                case 2:
                    $fix_encode = Chr(0x1B) . Chr(0x74) . Chr(0x07); // установка 7 кодовой страницы
                    break;
                default:
                    $fix_encode = Chr(0x1B) . Chr(0x74) . Chr(0x07);
                    break;
            }
        }

        $this->log(__METHOD__ . " Using COM-port: {$device->PortNumber}:{$device->BaudRate} (settings: {$this->_deviceCOMPortNumber}:{$this->_deviceCOMPortBaud})...\n", 3);
        $device->Enable = 1;

        //Исправление кодировки
        //$fix_encode = Chr(0x1B).Chr(0x3D).Chr(0x02).Chr(0x1B).Chr(0x74).Chr(0x06).Chr(0x1B).Chr(0x52).Chr(0x00);
        if (isset($fix_encode)) {
            $device->SendLine($fix_encode);
        }

        if ($device->ResultCode != 0) {
            $this->log(__METHOD__ . " ----- ERROR connecting \n", 3);
            return false;
        }
        $this->_connected = true;
        $this->log(__METHOD__ . " ----- Connected OK\n", 3);
        return true;
    }

    protected function _internalClose($dev = NULL) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = isset($dev) ? $dev : $this->get_NativeDevice();
        $this->log(__METHOD__ . " Disconnecting from device...\n", 3);
        $device->Enable = 0;
        if ($device->ResultCode != 0)
            return false;
        $this->_connected = false;
        $this->log(__METHOD__ . " Disonnected OK\n", 3);
        return true;
    }

    protected function _internalGetLastError($dev = NULL) {
        $device = isset($dev) ? $dev : $this->get_NativeDevice();
        return $device->ResultCode;
    }

    protected function _internalGetLastErrorMessage($dev = NULL) {
        $device = isset($dev) ? $dev : $this->get_NativeDevice();
        return iconv('windows-1251', 'utf-8', $device->ResultCodeDescriptor);
    }

    /* OVERRIDE DEVICE METHODS */

    /**
     * Устанавливает соединение с устройством для дальнейшей работы (открывает COM порт)
     * 
     * @param
     * @return bool
     */
    public function Open() {
        parent::Open();
        $result = $this->_internalOpen();
        if (!$result)
            return $this->_isDeviceFailed(__METHOD__);
        return array('success' => true);
    }

    /**
     * Закрывает соединение с устройством (закрывает COM порт)
     * 
     * @param
     * @return bool
     */
    public function Close() {
        parent::Close();
        $result = $this->_internalClose();
        if (!$result)
            return $this->_isDeviceFailed(__METHOD__);
        return array('success' => true);
    }

    /**
     * Возвращает ИСТИНА если соединение с устройством установлено
     * 
     * @param
     * @return bool
     */
    public function IsOpen() {
        $result = $this->_internalIsOpen();
        return array('success' => true, 'isopen' => $result);
    }

    /**
     * Инициализирует дисплей
     * 
     * @param array $params Массив параметров
     * @return bool
     */
    protected function _do_init($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();
        $device->InitialDispl();
        if ($errResult = $this->_isDeviceFailed(__METHOD__))
            return $errResult;
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return array('success' => true);
    }

    /**
     * Очищает дисплей
     * 
     * @param array $params Массив параметров
     * @return bool
     */
    protected function _do_clear($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();
        $device->ClearDispl();
        if ($errResult = $this->_isDeviceFailed(__METHOD__))
            return $errResult;
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return array('success' => true);
    }

    private function alignString($str) {
        $XSize = $this->XSize;
        $postfix = '';

        if (strpos($str, '|') !== false) {
            $elements = explode('|', $str, 2);
            if ($elements) {
                $str = $elements[0];
                $postfix = substr(' ' . $elements[1], 0, $XSize);
                $XSize -= strlen($postfix);
            }
        }

        $str = substr($str, 0, $XSize);
        $str = str_pad($str, $XSize) . $postfix;
        return $str;
    }

    /**
     * Выводит на дисплей строку. 
     * 
     * @param array $params Массив параметров. 
     *    Если передан только line1, то он выводится на девайс как есть без каких-либо преобразований
     *    Если кроме line1 присутствует line2, то line1 выводится на первой строке, line2 на второй
     *    Если в line1 или line2 присутствует символ | , то все что до этого символа прижимается к левому краю, а то, что правее символа - к правому
     * @return bool
     */
    protected function _do_print($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $line1 = $params['line1'];
        $line1Pos = isset($params['line1_pos']) ? $params['line1_pos'] : 0;
        $line2Pos = isset($params['line2_pos']) ? $params['line2_pos'] : $this->XSize;
        if (is_string($line1)) {
            $line1 = iconv('UTF-8', 'Windows-1251', $params['line1']);
            $line2 = '';
            if (isset($params['line2'])) {
                $line2 = iconv('UTF-8', 'Windows-1251', $params['line2']);
                $line1 = $this->alignString($line1);
                $line2 = $this->alignString($line2);
            }
            $device = $this->get_NativeDevice();
            //Выводит на дисплей строку(Строка), указанную во втором параметре, с позиции, указанной в НомерПозиции
            $device->ClearDispl();
            if ($errResult = $this->_isDeviceFailed(__METHOD__))
                return $errResult;
            $device->EnterStr($line1Pos, $line1);
            if ($errResult = $this->_isDeviceFailed(__METHOD__))
                return $errResult;
            if (!empty($line2)) {
                $device->EnterStr($line2Pos, $line2);
                if ($errResult = $this->_isDeviceFailed(__METHOD__))
                    return $errResult;
            }
        } else {
            $this->log(__METHOD__ . " ----- end Fail. Параметр line1 должен содержать строку\n", 1);
            return array('success' => false, 'error' => 'Параметр [line1] должен содержать строку', 'time' => time());
        }
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return array('success' => true, 'time' => time());
    }

    /**
     * Выводит на дисплей строку
     * 
     * @param array $params Массив параметров
     * @return bool
     */
    protected function _do_test($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();
        //Вывод на экран сообщения о скорости обмена и номере СОМ порта  для визуальной проверки наличия связи с устройством
        $device->Test();
        if ($errResult = $this->_isDeviceFailed(__METHOD__))
            return $errResult;
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return array('success' => true, 'time' => time());
    }

    /**
     * Прямой ввод строки символов. Может использоваться для прямого ввода команд протокола дисплея
     * 
     * @param array $params Массив параметров
     * @return bool
     */
    protected function _do_sendline($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $str = $params['string'];
        if (is_string($str)) {
            $device = $this->get_NativeDevice();
            //Прямой ввод строки символов. Может использоваться для прямого ввода команд протокола дисплея
            $device->SendLine($str);
            if ($errResult = $this->_isDeviceFailed(__METHOD__))
                return $errResult;
        }
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return array('success' => true);
    }

    /*     * *************************************
     * * PUBLIC METHODS 
     * *************************************** */

    /**
     * Возвращает версию драйвера
     * 
     * @param 
     * @return 
     */
    public function GetVersion() {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();

        $result = $device->Version;
        if ($errResult = $this->_isDeviceFailed(__METHOD__))
            return $errResult;
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return array('success' => true, 'version' => $result, 'time' => time());
    }

    /**
     * Возвражает размеры экрана
     * 
     * @param 
     * @return 
     */
    public function GetSize($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        if ($errResult = $this->_isDeviceFailed(__METHOD__))
            return $errResult;
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return array('success' => true, 'x' => $this->XSize, 'y' => $this->YSize, 'time' => time());
    }

}
