<?php

require_once 'core/OLEControlWrapper.php';

/**
 * Class FPAtol10Native
 * COM интерфейс драйвера АТОЛ версии 10
 * @link http://integration.atol.ru/api/
 *
 * Общие настройки драйвера:
 *
 * @property string LIBFPTR_SETTING_LIBRARY_PATH - путь к каталогу с библиотеками-зависимостями, подгружаемым
 * по-надобности. К таким библиотекам относятся libusb и libudev для работы с ККТ по USB на Linux. Может быть списком
 * каталогов, аналогично java.library.path. Не актуально для случая, когда используется драйвер, установленный с
 * помощью дистрибутива.
 *
 * @property string LIBFPTR_SETTING_MODEL - код модели ККТ. Определяет способ работы драйвера с устройством.
 * Может принимать следующие значения:
 * @property string LIBFPTR_MODEL_ATOL_AUTO - автоматическое определение модели (только ККТ АТОЛ)
 * @property string LIBFPTR_MODEL_ATOL_1F - АТОЛ 1Ф
 * @property string LIBFPTR_MODEL_ATOL_11F - АТОЛ 11Ф
 * @property string LIBFPTR_MODEL_ATOL_15F - АТОЛ 15Ф
 * @property string LIBFPTR_MODEL_ATOL_20F - АТОЛ 20Ф
 * @property string LIBFPTR_MODEL_ATOL_22F - АТОЛ 22Ф (АТОЛ FPrint-22ПТК)
 * @property string LIBFPTR_MODEL_ATOL_25F - АТОЛ 25Ф
 * @property string LIBFPTR_MODEL_ATOL_27F - АТОЛ 27Ф
 * @property string LIBFPTR_MODEL_ATOL_30F - АТОЛ 30Ф
 * @property string LIBFPTR_MODEL_ATOL_42FS - АТОЛ 42ФС
 * @property string LIBFPTR_MODEL_ATOL_50F - АТОЛ 50Ф
 * @property string LIBFPTR_MODEL_ATOL_52F - АТОЛ 52Ф
 * @property string LIBFPTR_MODEL_ATOL_55F - АТОЛ 55Ф
 * @property string LIBFPTR_MODEL_ATOL_60F - АТОЛ 60Ф
 * @property string LIBFPTR_MODEL_ATOL_77F - АТОЛ 77Ф
 * @property string LIBFPTR_MODEL_ATOL_90F - АТОЛ 90Ф
 * @property string LIBFPTR_MODEL_ATOL_91F - АТОЛ 91Ф
 * @property string LIBFPTR_MODEL_ATOL_92F - АТОЛ 92Ф
 * @property string LIBFPTR_MODEL_ATOL_SIGMA_10 - АТОЛ Sigma 10
 * @property string LIBFPTR_MODEL_ATOL_SIGMA_7F - АТОЛ Sigma 7Ф
 * @property string LIBFPTR_MODEL_ATOL_SIGMA_8F - АТОЛ Sigma 8Ф
 * @property string LIBFPTR_MODEL_KAZNACHEY_FA - Казначей ФА
 *
 * @property string LIBFPTR_SETTING_ACCESS_PASSWORD - пароль доступа ККТ. Требуется для взаимодействия с ККТ. Если не
 * указан или пуст, используется стандартный пароль в зависимости от настройки LIBFPTR_SETTING_MODEL.
 *
 * @property string LIBFPTR_SETTING_USER_PASSWORD - пароль пользователя по умолчанию. Требуется для доступа к
 * специфичным командам и режимам ККТ. Если не указан или пуст, используется пароль по умолчанию с максимальными правами
 * в зависимости от настройки LIBFPTR_SETTING_MODEL.
 *
 * @property string LIBFPTR_SETTING_OFD_CHANNEL - канал для обмена с ОФД. По умолчанию - LIBFPTR_OFD_CHANNEL_NONE.
 * Для корректной работы требуется дополнительная настройка ККТ (настройка #276, см. Настройки ККТ).
 * Возможные значения:
 * @property string LIBFPTR_OFD_CHANNEL_NONE - не активировать обмен с ОФД средствами драйвера.
 * @property string LIBFPTR_OFD_CHANNEL_PROTO - предоставить для ККТ канал обмена с ОФД через транспортный протокол ККТ.
 * @property string LIBFPTR_SETTING_SCRIPTS_PATH - путь к внешним, пользовательским, скриптам (подробнее в Скрипты драйвера).
 *
 * @property string LIBFPTR_SETTING_USE_DOCUMENTS_JOURNAL - использовать БД документов. Подробнее в соответствующем разделе.
 *
 * @property string LIBFPTR_SETTING_DOCUMENTS_JOURNAL_PATH - путь к БД документов. Подробнее в соответствующем разделе.
 *
 * @property string LIBFPTR_SETTING_INVERT_CASH_DRAWER_STATUS - инвертировать состояние денежного ящика или нет.
 * По умолчанию - не инвертировать.
 *
 *
 * Настройки связи с ККТ:
 *
 * @property string LIBFPTR_SETTING_PORT - способ связи с ККТ. Остальные настройки связи зависят от значения этой
 * настройки. Может принимать следующие значения:
 *
 * @property string LIBFPTR_PORT_COM - работа с ККТ либо через физический COM-порт, либо через виртуальные COM-порт и
 * TTY-файл. Доступно для Windows, Linux и Android. К данному каналу также относятся настройки LIBFPTR_SETTING_BAUDRATE,
 * LIBFPTR_SETTING_BITS, LIBFPTR_SETTING_STOPBITS, LIBFPTR_SETTING_PARITY, LIBFPTR_SETTING_COM_FILE.
 *
 * @property string LIBFPTR_PORT_USB - работа с ККТ по USB. Доступно для Windows, Linux, macOS и Android. К данному
 * каналу также относится настройка LIBFPTR_SETTING_USB_DEVICE_PATH.
 *
 * @property string LIBFPTR_PORT_TCPIP - работа с ККТ по TCP/IP (Ethernet, Wi-Fi). Доступно для Windows, Linux, macOS,
 * Android и iOS. К данному каналу также относятся настройки LIBFPTR_SETTING_IPADDRESS и LIBFPTR_SETTING_IPPORT.
 *
 * @property string LIBFPTR_PORT_BLUETOOTH - работа с ККТ по Bluetooth. Доступно для Windows, Linux и Android. К данному
 * каналу также относится настройка LIBFPTR_SETTING_MACADDRESS.
 *
 * @property string LIBFPTR_SETTING_BAUDRATE - скорость работы ККТ по RS232.
 * Может принимать следующие значения:
 * @property string LIBFPTR_PORT_BR_1200 - 1200 бод
 * @property string LIBFPTR_PORT_BR_2400 = 2400 бод
 * @property string LIBFPTR_PORT_BR_4800 = 4800 бод
 * @property string LIBFPTR_PORT_BR_9600 = 9600 бод
 * @property string LIBFPTR_PORT_BR_19200 = 19200 бод
 * @property string LIBFPTR_PORT_BR_38400 = 38400 бод
 * @property string LIBFPTR_PORT_BR_57600 = 57600 бод
 * @property string LIBFPTR_PORT_BR_115200 = 115200 бод
 * @property string LIBFPTR_PORT_BR_230400 = 230400 бод
 * @property string LIBFPTR_PORT_BR_460800 = 460800 бод
 * @property string LIBFPTR_PORT_BR_921600 = 921600 бод
 *
 * @property string LIBFPTR_SETTING_BITS - количество бит в байте канала RS232.
 * @property string LIBFPTR_PORT_BITS_7 - 7 бит
 * @property string LIBFPTR_PORT_BITS_8 - 8 бит
 *
 * @property string LIBFPTR_SETTING_STOPBITS - количество стоп-битов канала RS232
 * @property string LIBFPTR_PORT_SB_1 - 1 бит
 * @property string LIBFPTR_PORT_SB_1_5 - 1.5 бита
 * @property string LIBFPTR_PORT_SB_2 - 2 бита
 *
 * @property string LIBFPTR_SETTING_PARITY - четность канала RS232
 * @property string LIBFPTR_PORT_PARITY_NO - нет
 * @property string LIBFPTR_PORT_PARITY_ODD - четность
 * @property string LIBFPTR_PORT_PARITY_EVEN - нечетность
 * @property string LIBFPTR_PORT_PARITY_MARK - установлена
 * @property string LIBFPTR_PORT_PARITY_SPACE - сброшена
 *
 * @property string LIBFPTR_SETTING_COM_FILE - идентификатор порта. Для Windows - номер COM-порта (например, "15" или "COM15").
 * Для Linux - путь к TTY-файлу (например, "/dev/ttyS5" или "/dev/ttyACM0").
 *
 * @property string LIBFPTR_SETTING_USB_DEVICE_PATH - расположение USB-устройства в системе (только Linux).
 * Если настройка не указана, имеет значение "auto" или пустое значение, то поиск и подключение к устройству будет
 * проводиться по известным драйверу VendorID и ProductID. На Windows подключение всегда ведется по VendorID и ProductID.
 *
 * @property string LIBFPTR_SETTING_IPADDRESS - IP-адрес ККТ. Может быть как ip-адресом, так и hostname-ом.
 * @property string LIBFPTR_SETTING_IPPORT - IP-порт ККТ.
 * @property string LIBFPTR_SETTING_MACADDRESS - Bluetooth MAC-адрес ККТ. Устройство должно быть предварительно
 * сопряжено с хостом средствами ОС.
 * @property string LIBFPTR_SETTING_AUTO_RECONNECT - автоматически восстанавливать связь с ККТ при обрыве соединения. В
 * случае, если значение false, драйвер не будет автоматически пытаться восстановить связь после обнаружения отключения
 * устройства или получения ошибки "Нет связи", а будет сразу возвращать ошибку LIBFPTR_ERROR_CONNECTION_LOST (Соединение с ККТ потеряно).
 * Значение по умолчанию - true.
 *
 *
 *
 *
 * @property string LIBFPTR_PARAM_DATA_TYPE - тип запроса к ККТ
 * @property string LIBFPTR_DT_STATUS
 * @property string LIBFPTR_DT_SHORT_STATUS
 * @property string LIBFPTR_DT_DATE_TIME - текущая дата
 * @property string LIBFPTR_DT_SHIFT_STATE - состояние смены
 * @property string LIBFPTR_DT_SHIFT_TAX_SUM - сумма налога за смену
 * @property string LIBFPTR_DT_REVENUE - выручка
 * @property string LIBFPTR_DT_CASHIN_COUNT - количество внесений
 * @property string LIBFPTR_DT_CASHOUT_COUNT - количество выплат
 * @property string LIBFPTR_DT_RECEIPT_STATE - состояние чека
 *
 *
 * @property string LIBFPTR_PARAM_FN_DATA_TYPE - тип запроса к ФН
 * Может принимать значения:
 * @property string LIBFPTR_FNDT_TAG_VALUE
 * @property string LIBFPTR_FNDT_SHIFT - информация о смене
 * @property string LIBFPTR_FNDT_FN_INFO - информация о ФН
 * @property string LIBFPTR_FNDT_LAST_DOCUMENT - информация о последнем документе
 * @property string LIBFPTR_FNDT_REG_INFO
 *
 *
 *
 * @property string LIBFPTR_PARAM_SERIAL_NUMBER Серийный номер
 * @property string LIBFPTR_PARAM_MODEL_NAME Название ККТ
 * @property string LIBFPTR_PARAM_UNIT_VERSION Версия ПО ККТ
 *
 *
 * @property string LIBFPTR_PARAM_DOCUMENT_NUMBER
 * @property int LIBFPTR_PARAM_DOCUMENTS_COUNT
 * @property int LIBFPTR_PARAM_DOCUMENT_CLOSED
 *
 * @property int LIBFPTR_PARAM_REPORT_TYPE - тип отчета
 * Может принимать значения:
 * @property int LIBFPTR_RT_X - x отчет
 * @property int LIBFPTR_RT_CLOSE_SHIFT - z отчет
 * @property int LIBFPTR_RT_LAST_DOCUMENT - копия последнего документа
 * @property int LIBFPTR_RT_OFD_EXCHANGE_STATUS - отчет о состоянии расчетов
 * @property int LIBFPTR_RT_KKT_DEMO - демо печать
 * @property int LIBFPTR_RT_KKT_INFO - печать информации о ККТ
 * @property int LIBFPTR_RT_OFD_TEST печать лиагностики соединения с ОФД
 * @property int LIBFPTR_RT_FN_DOC_BY_NUMBER печать документа из архива фискального накопителя
 * @property int LIBFPTR_RT_QUANTITY печать отчета количеств
 * @property int LIBFPTR_RT_DEPARTMENTS печать отчета по секциям
 * @property int LIBFPTR_RT_OPERATORS печать отчета по кассирам
 * @property int LIBFPTR_RT_HOURS печать отчета по часам
 * @property int LIBFPTR_RT_FN_REGISTRATIONS печать итогов регистрации/перерегистрации
 * @property int LIBFPTR_RT_FN_SHIFT_TOTAL_COUNTERS печать счетчиков итогов смены
 * @property int LIBFPTR_RT_FN_TOTAL_COUNTERS печать счетчиков итогов ФН
 * @property int LIBFPTR_RT_FN_NOT_SENT_DOCUMENTS_COUNTERS печать счетчиков по непереданным документам
 * @property int LIBFPTR_RT_COMMODITIES_BY_TAXATION_TYPES печать отчета по товарам по СНО
 * @property int LIBFPTR_RT_COMMODITIES_BY_DEPARTMENTS печать отчета по товарам по отделам
 * @property int LIBFPTR_RT_COMMODITIES_BY_SUMS печать отчета по товарам по суммам
 * @property int LIBFPTR_RT_START_SERVICE печать служебного отчета
 * @property int LIBFPTR_RT_DISCOUNTS печать отчета по скидкам
 *
 * @property int LIBFPTR_PARAM_RECEIPT_TYPE - тип чека
 * Может принимать значения:
 * @property int LIBFPTR_RT_CLOSED - чек закрыт
 * @property int LIBFPTR_RT_SELL - чек прихода (продажи)
 * @property int LIBFPTR_RT_SELL_RETURN - чек возврата прихода (продажи)
 * @property int LIBFPTR_RT_SELL_CORRECTION - чек коррекции прихода
 * @property int LIBFPTR_RT_SELL_RETURN_CORRECTION - чек коррекции возврата прихода
 * @property int LIBFPTR_RT_BUY - чек расхода (покупки)
 * @property int LIBFPTR_RT_BUY_RETURN - чек возврата расхода (покупки)
 * @property int LIBFPTR_RT_BUY_CORRECTION - чек коррекции расхода
 * @property int LIBFPTR_RT_BUY_RETURN_CORRECTION - чек коррекции возврата расхода
 *
 * @property string LIBFPTR_PARAM_RECEIPT_NUMBER
 * @property string LIBFPTR_PARAM_FISCAL_SIGN
 *
 * @property int LIBFPTR_PARAM_PAYMENT_TYPE способ оплаты
 * Может принимать значения:
 * @property int LIBFPTR_PT_CASH - наличными
 * @property int LIBFPTR_PT_ELECTRONICALLY - безналичными
 * @property int LIBFPTR_PT_PREPAID - предварительная оплата (аванс)
 * @property int LIBFPTR_PT_CREDIT - последующая оплата (кредит)
 * @property int LIBFPTR_PT_OTHER - иная форма оплаты (встречное предоставление)
 * @property int LIBFPTR_PT_6 - способ расчета №6
 * @property int LIBFPTR_PT_7 - способ расчета №7
 * @property int LIBFPTR_PT_8 - способ расчета №8
 * @property int LIBFPTR_PT_9 - способ расчета №9
 * @property int LIBFPTR_PT_10 - способ расчета №10
 *
 * @property float LIBFPTR_PARAM_PAYMENT_SUM сумма оплаты
 * @property float LIBFPTR_PARAM_RECEIPT_SUM сумма чека
 * @property float LIBFPTR_PARAM_REMAINDER Неоплаченный остаток чека
 * @property float LIBFPTR_PARAM_CHANGE сСдача по чеку
 *
 * @property int LIBFPTR_PARAM_SHIFT_STATE - состояние смены:
 * @property int LIBFPTR_SS_CLOSED - смена закрыта
 * @property int LIBFPTR_SS_OPENED - смена открыта
 * @property int LIBFPTR_SS_EXPIRED - смена истекла (продолжительность смены больше 24 часов)
 * @property string LIBFPTR_PARAM_SHIFT_NUMBER
 *
 * @property string LIBFPTR_PARAM_DATE_TIME
 *
 * @property string LIBFPTR_PARAM_TAG_NAME
 * @property string LIBFPTR_PARAM_TAG_TYPE
 * @property string LIBFPTR_PARAM_TAG_NUMBER
 * @property string LIBFPTR_PARAM_TAG_VALUE
 *
 * @property int LIBFPTR_PARAM_TAX_MODE - способ начисления налога.
 * Параметр принимает следующие значения:
 * @property int LIBFPTR_TM_POSITION - на позицию целиком
 * @property int LIBFPTR_TM_UNIT - на единицу товара
 *
 * @property float LIBFPTR_PARAM_TAX_SUM - сумма налога. При передаче значения 0 рассчитывается автоматически
 * @property int LIBFPTR_PARAM_TAX_TYPE
 * Может принимать значения:
 * @property int LIBFPTR_TAX_VAT18 - НДС 18%
 * @property int LIBFPTR_TAX_VAT10 - НДС 10%
 * @property int LIBFPTR_TAX_VAT118 - НДС расчитанный 18/118
 * @property int LIBFPTR_TAX_VAT110 - НДС расчитанный 10/110
 * @property int LIBFPTR_TAX_VAT0 - НДС 0%
 * @property int LIBFPTR_TAX_NO - не облагается
 * @property int LIBFPTR_TAX_VAT20 - НДС 20%
 * @property int LIBFPTR_TAX_VAT120 - НДС расчитанный 20/120
 *
 * @property string LIBFPTR_PARAM_FREQUENCY
 * @property string LIBFPTR_PARAM_DURATION
 *
 * @property int LIBFPTR_PARAM_OPERATOR_ID Номер кассира
 * @property int LIBFPTR_PARAM_LOGICAL_NUMBER Номер ККТ в магазине
 * @property int LIBFPTR_PARAM_MODEL Номер модели ККТ
 * @property int LIBFPTR_PARAM_MODE Режим ККТ
 * @property int LIBFPTR_PARAM_SUBMODE Подрежим ККТ
 * @property int LIBFPTR_PARAM_RECEIPT_LINE_LENGTH Ширина чековой ленты, симв.
 * @property int LIBFPTR_PARAM_RECEIPT_LINE_LENGTH_PIX Ширина чековой ленты, пикс.
 * @property int LIBFPTR_PARAM_FISCAL Флаг регистрации ККТ
 * @property int LIBFPTR_PARAM_FN_FISCAL Флаг фискализации ФН
 * @property int LIBFPTR_PARAM_FN_PRESENT Флаг наличия ФН в ККТ
 * @property int LIBFPTR_PARAM_INVALID_FN Флаг корректности ФН
 * @property int LIBFPTR_PARAM_CASHDRAWER_OPENED Денежный ящик открыт
 * @property int LIBFPTR_PARAM_RECEIPT_PAPER_PRESENT Наличие бумаги
 * @property int LIBFPTR_PARAM_PAPER_NEAR_END Бумага скоро закончится. Если датчика скорого окончания бумаги в ККТ нет,
 * то будет всегда возвращаться false
 * @property int LIBFPTR_PARAM_COVER_OPENED Крышка открыта
 * @property int LIBFPTR_PARAM_PRINTER_CONNECTION_LOST Потеряно соединение с печатным механизмом
 * @property int LIBFPTR_PARAM_PRINTER_ERROR Невосстановимая ошибка печатного механизма
 * @property int LIBFPTR_PARAM_CUT_ERROR Ошибка отрезчика
 * @property int LIBFPTR_PARAM_PRINTER_OVERHEAT Перегрев печатного механизма
 * @property int LIBFPTR_PARAM_BLOCKED ККТ заблокирована из-за ошибок
 *
 * @property int LIBFPTR_PARAM_JSON_DATA
 *
 * @property int LIBFPTR_PARAM_CUT_TYPE - тип отрезки:
 * @property int LIBFPTR_CT_FULL - полная
 * @property int LIBFPTR_CT_PART - частичная
 *
 * @property int LIBFPTR_PARAM_DRAWER_ON_TIMEOUT - длительность включения, мс.
 * @property int LIBFPTR_PARAM_DRAWER_OFF_TIMEOUT - длительность задержки, мс.
 * @property int LIBFPTR_PARAM_DRAWER_ON_QUANTITY - количество импульсов
 *
 * @property float LIBFPTR_PARAM_SUM
 *
 * @property string LIBFPTR_PARAM_COMMODITY_NAME - название товара
 * @property float LIBFPTR_PARAM_PRICE - цена за единицу
 * @property float LIBFPTR_PARAM_QUANTITY - количество единиц товара
 * @property int LIBFPTR_PARAM_DEPARTMENT - номер отдела
 * @property float LIBFPTR_PARAM_POSITION_SUM - полная сумма позиции. Может отличаться от произведения цены на
 * количество. В этом случае ККТ разобьет позицию на две, распределив получившуюся разницу (скидку или надбавку) между
 * позициями.
 *
 * @property string LIBFPTR_PARAM_BARCODE
 * @property int LIBFPTR_PARAM_BARCODE_TYPE - тип штрихкода
 * Может принимать следующие знаечения:
 *  Одномерные штрихкоды
 * @property int LIBFPTR_BT_EAN_8 - EAN-8
 * @property int LIBFPTR_BT_EAN_13 - EAN-13
 * @property int LIBFPTR_BT_UPC_A - UPC-A
 * @property int LIBFPTR_BT_UPC_E - UPC-E
 * @property int LIBFPTR_BT_CODE_39 - Code 39
 * @property int LIBFPTR_BT_CODE_93 - Code 93
 * @property int LIBFPTR_BT_CODE_128 - Code 128
 * @property int LIBFPTR_BT_CODABAR - Codabar
 * @property int LIBFPTR_BT_ITF - Interleaved 2of5
 * @property int LIBFPTR_BT_ITF_14 - ITF-14
 * @property int LIBFPTR_BT_GS1_128 - GS1-128 (EAN-128)
 * @property int LIBFPTR_BT_CODE_39_EXTENDED - Code 39 Extended
 *  Двумерные штрихкоды
 * @property int LIBFPTR_BT_QR - QR-код
 * @property int LIBFPTR_BT_PDF417 - PDF417
 * @property int LIBFPTR_BT_AZTEC - AZTEC
 *
 * @property int LIBFPTR_PARAM_SCALE Коэффициент увеличения штрихкода
 * @property int LIBFPTR_PARAM_LEFT_MARGIN Дополнительный отступ слева
 * @property bool LIBFPTR_PARAM_BARCODE_INVERT Инверсия цвета
 * Для одномерных штрихкодов дополнительно можно задать следующие параметры:
 * @property int LIBFPTR_PARAM_HEIGHT Высота штрихкода, пикс.
 * @property bool LIBFPTR_PARAM_BARCODE_PRINT_TEXT Флаг печати данных ШК
 * Для QR-кода дополнительно можно задать следующие параметры:
 * @property int LIBFPTR_PARAM_BARCODE_CORRECTION Коррекция QR-кода
 * Может принимать следующие значения:
 * @property int LIBFPTR_BC_DEFAULT - по умолчанию
 * @property int LIBFPTR_BC_0 - минимальная
 * @property int LIBFPTR_BC_1
 * @property int LIBFPTR_BC_2
 * @property int LIBFPTR_BC_3 - максимальная для QR / AZTEC
 * @property int LIBFPTR_BC_4
 * @property int LIBFPTR_BC_5
 * @property int LIBFPTR_BC_6
 * @property int LIBFPTR_BC_7
 * @property int LIBFPTR_BC_8 - максимальная для PDF-417
 *
 * @property int LIBFPTR_PARAM_BARCODE_VERSION Версия QR-кода
 * @property int LIBFPTR_PARAM_BARCODE_COLUMNS	Количество столбцов PDF417
 *
 *
 * @property string LIBFPTR_PARAM_TEXT - Строка для печати
 * @property int LIBFPTR_PARAM_ALIGNMENT - Выравнивание
 * Может принимать значения:
 * @property int LIBFPTR_ALIGNMENT_LEFT - по левому краю (по умолчанию)
 * @property int LIBFPTR_ALIGNMENT_CENTER - по центру
 * @property int LIBFPTR_ALIGNMENT_RIGHT - по правому краю
 *
 * @property int LIBFPTR_PARAM_TEXT_WRAP - Перенос строки
 * Может принимать значения:
 * @property int LIBFPTR_TW_NONE - не переносить. Текст будет обрезаться по ширине ленты (по умолчанию)
 * @property int LIBFPTR_TW_WORDS - переносить по словам. Разделителями считаются символы:
 * пробел
 * точка
 * запятая
 * точка с запятой
 * тире
 * знак вопроса
 * восклицательный знак
 * закрывающая скобка
 * закрывающая фигурная скобка
 * закрывающая квадратная скобка
 * @property int LIBFPTR_TW_CHARS - переносить по символам
 *
 * @property int LIBFPTR_PARAM_FONT	- Номер шрифта
 * @property bool LIBFPTR_PARAM_FONT_DOUBLE_WIDTH - Двойная ширина. По умолчанию false
 * @property bool LIBFPTR_PARAM_FONT_DOUBLE_HEIGHT - Двойная высота. По умолчанию false
 * @property int LIBFPTR_PARAM_LINESPACING - Межстрочный интервал
 * @property int LIBFPTR_PARAM_BRIGHTNESS - Яркость
 * @property bool LIBFPTR_PARAM_STORE_IN_JOURNAL - Сохранять в электронном журнале
 *
 * Значения реквизита 1055 - Применяемая система налогообложения (СНО)
 * @property int LIBFPTR_TT_OSN - общая
 * @property int LIBFPTR_TT_USN_INCOME - упрощенная доход
 * @property int LIBFPTR_TT_USN_INCOME_OUTCOME - упрощенная доход минус расход
 * @property int LIBFPTR_TT_ENVD - единый налог на вмененный доход
 * @property int LIBFPTR_TT_ESN - единый сельскохозяйственный доход
 * @property int LIBFPTR_TT_PATENT - патентная система налогообложения
 *
 * @property int LIBFPTR_PARAM_MARKING_CODE_TYPE
 * @property int LIBFPTR_MCT12_AUTO
 * @property int LIBFPTR_PARAM_MARKING_CODE
 * @property int LIBFPTR_PARAM_MARKING_CODE_STATUS
 * @property int LIBFPTR_PARAM_MEASUREMENT_UNIT
 * @property int LIBFPTR_IU_PIECE
 * @property int LIBFPTR_PARAM_MARKING_WAIT_FOR_VALIDATION_RESULT
 * @property int LIBFPTR_PARAM_MARKING_PROCESSING_MODE
 * @property int LIBFPTR_PARAM_MARKING_CODE_VALIDATION_READY
 * @property int LIBFPTR_PARAM_MARKING_CODE_ONLINE_VALIDATION_RESULT
 * @property int LIBFPTR_PARAM_IS_REQUEST_SENT
 * @property int LIBFPTR_PARAM_MARKING_CODE_ONLINE_VALIDATION_ERROR
 * @property int LIBFPTR_PARAM_MARKING_CODE_ONLINE_VALIDATION_ERROR_DESCRIPTION
 *
 * Методы:
 * @method getSingleSetting($param)
 * @method setSingleSetting($param, $value)
 * @method applySingleSettings()
 *
 * @method setParam($param, $value)
 * @method setParamStrHex($param, $value) Устанавливает значение параметра типа bytearray принимая на вход строку с
 * последовательностью байтов вида "01 02 FF AA"
 * @method setNonPrintableParam($param, $value)
 * @method getParamString($param): string
 * @method getParamInt($param): int
 * @method getParamDouble($param): float
 * @method getParamBool($param): bool
 * @method getParamDateTime($param): variant object
 * @method getParamByteArray($param): variant object
 * @method utilFormTlv()
 *
 * @method fnQueryData() Возвращает данные из ФН
 * @method queryData() Возвращает данные из ККТ
 *
 * @method operatorLogin()
 * @method openShift()
 * @method checkDocumentClosed()
 *
 * @method open()
 * @method isOpened(): bool
 * @method close()
 *
 * @method beep()
 *
 * @method processJson() Выполняет json-задание
 *
 * @method cut() Отрезка ленты
 * @method openDrawer() Открывает денежный ящик
 * @method cashIncome() Внесение
 * @method cashOutcome() Выплата
 * @method payment() Зарегистрировать оплату
 *
 * @method openReceipt() Открыть чек
 * @method registration() Регистрация позиции
 * @method receiptTotal() Регистрация итога
 * @method closeReceipt() Закрыть чек
 * @method continuePrint() Допечатать документ
 * @method cancelReceipt() Отменить чек
 * @method report() Печать отчета
 *
 * @method printText() Печать текста
 * @method printBarcode() Печать штрихкода
 *
 * @method beginMarkingCodeValidation()
 * @method getMarkingCodeValidationStatus()
 * @method cancelMarkingCodeValidation()
 * @method acceptMarkingCode()
 *
 * Информация о последней ошибке:
 * @method errorCode()
 * @method errorDescription()
 */
class FPAtol10Native extends OLEControlWrapper
{
    // COM класс драйвера
    const OLE_FISCAL_PRINTER_ATOL_DRIVER_10_PROGID = 'AddIn.Fptr10';

    // Скорости порта
    const LIBFPTR_PORT_BR_1200 = 'LIBFPTR_PORT_BR_1200';
    const LIBFPTR_PORT_BR_2400 = 'LIBFPTR_PORT_BR_2400';
    const LIBFPTR_PORT_BR_4800 = 'LIBFPTR_PORT_BR_4800';
    const LIBFPTR_PORT_BR_9600 = 'LIBFPTR_PORT_BR_9600';
    const LIBFPTR_PORT_BR_19200 = 'LIBFPTR_PORT_BR_19200';
    const LIBFPTR_PORT_BR_38400 = 'LIBFPTR_PORT_BR_38400';
    const LIBFPTR_PORT_BR_57600 = 'LIBFPTR_PORT_BR_57600';
    const LIBFPTR_PORT_BR_115200 = 'LIBFPTR_PORT_BR_115200';
    const LIBFPTR_PORT_BR_230400 = 'LIBFPTR_PORT_BR_230400';
    const LIBFPTR_PORT_BR_460800 = 'LIBFPTR_PORT_BR_460800';
    const LIBFPTR_PORT_BR_921600 = 'LIBFPTR_PORT_BR_921600';

    // Типы чеков
    const LIBFPTR_RT_SELL = 'LIBFPTR_RT_SELL';
    const LIBFPTR_RT_SELL_RETURN = 'LIBFPTR_RT_SELL_RETURN';
    const LIBFPTR_RT_SELL_CORRECTION = 'LIBFPTR_RT_SELL_CORRECTION';
    const LIBFPTR_RT_SELL_RETURN_CORRECTION = 'LIBFPTR_RT_SELL_RETURN_CORRECTION';
    const LIBFPTR_RT_BUY = 'LIBFPTR_RT_BUY';
    const LIBFPTR_RT_BUY_RETURN = 'LIBFPTR_RT_BUY_RETURN';
    const LIBFPTR_RT_BUY_CORRECTION = 'LIBFPTR_RT_BUY_CORRECTION';
    const LIBFPTR_RT_BUY_RETURN_CORRECTION = 'LIBFPTR_RT_BUY_RETURN_CORRECTION';

    // Система налогобложения
    const LIBFPTR_TT_OSN = 'LIBFPTR_TT_OSN';
    const LIBFPTR_TT_USN_INCOME = 'LIBFPTR_TT_USN_INCOME';
    const LIBFPTR_TT_USN_INCOME_OUTCOME = 'LIBFPTR_TT_USN_INCOME_OUTCOME';
    const LIBFPTR_TT_ENVD = 'LIBFPTR_TT_ENVD';
    const LIBFPTR_TT_ESN = 'LIBFPTR_TT_ESN';
    const LIBFPTR_TT_PATENT = 'LIBFPTR_TT_PATENT';

    // НДС
    const LIBFPTR_TAX_VAT18 = 'LIBFPTR_TAX_VAT18';
    const LIBFPTR_TAX_VAT10 = 'LIBFPTR_TAX_VAT10';
    const LIBFPTR_TAX_VAT118 = 'LIBFPTR_TAX_VAT118';
    const LIBFPTR_TAX_VAT110 = 'LIBFPTR_TAX_VAT110';
    const LIBFPTR_TAX_VAT0 = 'LIBFPTR_TAX_VAT0';
    const LIBFPTR_TAX_NO = 'LIBFPTR_TAX_NO';
    const LIBFPTR_TAX_VAT20 = 'LIBFPTR_TAX_VAT20';
    const LIBFPTR_TAX_VAT120 = 'LIBFPTR_TAX_VAT120';

    // Типы штрихкодов
    const LIBFPTR_BT_EAN_8 = 'LIBFPTR_BT_EAN_8';
    const LIBFPTR_BT_EAN_13 = 'LIBFPTR_BT_EAN_13';
    const LIBFPTR_BT_UPC_A = 'LIBFPTR_BT_UPC_A';
    const LIBFPTR_BT_UPC_E = 'LIBFPTR_BT_UPC_E';
    const LIBFPTR_BT_CODE_39 = 'LIBFPTR_BT_CODE_39';
    const LIBFPTR_BT_CODE_93 = 'LIBFPTR_BT_CODE_93';
    const LIBFPTR_BT_CODE_128 = 'LIBFPTR_BT_CODE_128';
    const LIBFPTR_BT_CODABAR = 'LIBFPTR_BT_CODABAR';
    const LIBFPTR_BT_ITF = 'LIBFPTR_BT_ITF';
    const LIBFPTR_BT_ITF_14 = 'LIBFPTR_BT_ITF_14';
    const LIBFPTR_BT_GS1_128 = 'LIBFPTR_BT_GS1_128';
    const LIBFPTR_BT_CODE_39_EXTENDED = 'LIBFPTR_BT_CODE_39_EXTENDED';
    const LIBFPTR_BT_QR = 'LIBFPTR_BT_QR';
    const LIBFPTR_BT_PDF417 = 'LIBFPTR_BT_PDF417';
    const LIBFPTR_BT_AZTEC = 'LIBFPTR_BT_AZTEC';

    // Выравнивание текста
    const LIBFPTR_ALIGNMENT_CENTER = 'LIBFPTR_ALIGNMENT_CENTER';
    const LIBFPTR_ALIGNMENT_LEFT = 'LIBFPTR_ALIGNMENT_LEFT';
    const LIBFPTR_ALIGNMENT_RIGHT = 'LIBFPTR_ALIGNMENT_RIGHT';
}
