<?php

require 'core/pinpad.php';

/* This class use exe file instead of OLE */

class pinpad_ibox extends Pinpad {

    const PATH_TO_EXE = "ibox\\iboxConsole.exe";
    const IBOX_PREFIX = "ibox";
    const IBOX_MODEL_P17 = "p17";
    const EVENT_TIMEOUT = "EventTimeout";
    const EVENT_READER = "EventReader";
    const EVENT_PAYMENT_ERROR = "EventPaymentError";
    const EVENT_REVERSE = "EventReverse";
    const EVENT_PAYMENT_FINISHED = "PaymentFinished";
    const EVENT_TRANSACTION_STARTED = "TransactionStarted";
    const METHOD_ENABLE = "Enable";
    const METHOD_DISABLE = "Disable";
    const METHOD_SET_CREDENTIALS = "SetCredentials";
    const METHOD_START_PAYMENT = "StartPayment";
    const METHOD_START_REVERSE = "StartReverse";
    const KEY_SUCCESS = "success";
    const KEY_EVENT = "event";
    const KEY_MESSAGE = "message";
    const KEY_LOG = "log";
    const KEY_TRANSACTION_ID = "transactionID";
    const KEY_SLIP = "slip";
    const VALUE_EVENT_PAYMENT_CANCELED = "PaymentCanceled";

    /**
     * Список умений данного устройства
     * 
     */
    private $_actions = array(
        'pay' => array(),
        'payCansel' => array(),
    );
    
    /**
     * Оплата покупки
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:Pay&encode=TEXT&p={"amount":100}
     * 
     * @param type $params
     */
    protected function _do_pay($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        
        set_time_limit(5 * 60);

        if (!isset($params['amount'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_PARAM_ERROR,
                'error' => 'Во входящих параметрах не найдено [amount]',
            );
        }

        if (!is_int($params['amount'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_AMOUNT_MISTAKE,
                'error' => 'Параметр [amount] повреждён, он должен быть целочисленным и больше нуля',
            );
        }

        if (!isset($params['cr'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_PARAM_ERROR,
                'error' => 'Во входящих параметрах не найдены данные для авторизации агента ibox',
            );
        }

        $decodedCredentials = base64_decode($params['cr']);
        $credentialsParts = explode(":", $decodedCredentials, 2);
        if (count($credentialsParts) != 2) {
            return array(
                'success' => false,
                'errcode' => parent::EC_PARAM_ERROR,
                'error' => 'Данные для авторизации агента ibox повреждены',
            );
        }

        $amount = $params['amount'] / 100;
        $login = $credentialsParts[0];
        $password = $credentialsParts[1];
        
        $model = $this->_deviceModel == -1 ? self::IBOX_MODEL_P17 : $this->_deviceModel;
        $port = $this->_deviceCOMPortNumber == -1 ? null : 'com'.$this->_deviceCOMPortNumber;
        $command = $this->getPayCommand($login, $password, $amount, $model, $port);
        $executionResult = $this->executeCommand($command);
        
        $paymentSuccessValues = $this->getValue($executionResult, self::EVENT_PAYMENT_FINISHED, self::KEY_SUCCESS);
        $success = count($paymentSuccessValues) > 0 && in_array(1, $paymentSuccessValues);

        if ($success) {
            $slipValues = $this->getValue($executionResult, self::EVENT_PAYMENT_FINISHED, self::KEY_SLIP);
            if (count($slipValues) > 0) {
                $slipArray = $this->splitStringToArray($slipValues[0]);
                $this->modifyStringArrayAsSberbank($slipArray);
                $result['Cheque'] = $slipArray;
            }
            $transactionIdValues = $this->getValue($executionResult, self::EVENT_PAYMENT_FINISHED, self::KEY_TRANSACTION_ID);
            if (count($transactionIdValues) > 0) {
                $result['MerchantTSN'] = $transactionIdValues[0];
            }
        } else {
            $eventCanceledValues = $this->getValue($executionResult, self::EVENT_READER, self::KEY_EVENT);
            if (count($eventCanceledValues) > 0 && in_array(self::VALUE_EVENT_PAYMENT_CANCELED, $eventCanceledValues)) {
                $result['errorinfo'] = "Оплата была отменена пользователем";
            } else {
                $result['errorinfo'] = "Ошибка во время выполнения операции. Платёж не прошёл.";
            }
            $result['errornum'] = 1;
        }

        $this->log(__METHOD__ . " ----- end OK\n", 3);

        return compact('success', 'result');
    }
    
    /**
     * Отмена операции. Слово cansel написано так и распространено по всему VirtualPos по прихоти Артура Кривцова (wartur)
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:PayCansel&encode=TEXT&p={"amount":100}
     * 
     * @param type $params
     */
    protected function _do_payCansel($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        
        set_time_limit(5 * 60);

        $amount = isset($params['amount']) && is_int($params['amount'])? $params['amount'] : null;
        
        if (!isset($params['cr'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_PARAM_ERROR,
                'error' => 'Во входящих параметрах не найдены данные для авторизации агента ibox',
            );
        }
        
        if (!isset($params['transaction_id']) || empty($params['transaction_id'])) {
            $transactionId = "";
        } else {
            $transactionId = $params['transaction_id'];
        }
        
        $decodedCredentials = base64_decode($params['cr']);
        $credentialsParts = explode(":", $decodedCredentials, 2);
        if (count($credentialsParts) != 2) {
            return array(
                'success' => false,
                'errcode' => parent::EC_PARAM_ERROR,
                'error' => 'Данные для авторизации агента ibox повреждены',
            );
        }
        
        $amount = isset($amount) ? $amount / 100 : null;
        $login = $credentialsParts[0];
        $password = $credentialsParts[1];
        
        $model = $this->_deviceModel == -1 ? self::IBOX_MODEL_P17 : $this->_deviceModel;
        $port = $this->_deviceCOMPortNumber == -1 ? null : 'com' . $this->_deviceCOMPortNumber;
        $command = $this->getCancelCommand($login, $password, null, $amount, $model, $port);
        $executionResult = $this->executeCommand($command);

        $paymentSuccessValues = $this->getValue($executionResult, self::EVENT_PAYMENT_FINISHED, self::KEY_SUCCESS);
        $success = count($paymentSuccessValues) > 0 && in_array(1, $paymentSuccessValues);

        if ($success) {
            $slipValues = $this->getValue($executionResult, self::EVENT_PAYMENT_FINISHED, self::KEY_SLIP);
            if (count($slipValues) > 0) {
                $slipArray = $this->splitStringToArray($slipValues[0]);
                $this->modifyStringArrayAsSberbank($slipArray);
                $result['Cheque'] = $slipArray;
            }
            $transactionIdValues = $this->getValue($executionResult, self::EVENT_PAYMENT_FINISHED, self::KEY_TRANSACTION_ID);
            if (count($transactionIdValues) > 0) {
                $result['MerchantTSN'] = $transactionIdValues[0];
            }
        } else {
            $eventCanceledValues = $this->getValue($executionResult, self::EVENT_READER, self::KEY_EVENT);
            if (count($eventCanceledValues) > 0 && in_array(self::VALUE_EVENT_PAYMENT_CANCELED, $eventCanceledValues)) {
                $result['errorinfo'] = "Оплата была отменена пользователем";
            } else {
                $result['errorinfo'] = "Ошибка во время выполнения операции. Платёж не прошёл.";
            }
            $result['errornum'] = 1;
        }

        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }
    
    /**
     * Делает тоже самое, что и _do_payCansel. Добавлено для совместимости с терминалом сбербанка
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:PayTransactionCansel&encode=TEXT
     * 
     * @param type $params
     */
    protected function _do_payTransactionCansel($params) {
        return $this->_do_payCansel($params);
    }

    /**
     * Здесь должно быть закрытие смены, но ibox его не поддерживает, поэтому всегда возвращает true
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:CloseShift&encode=TEXT
     * 
     * @param type $params
     * @return type
     */
    protected function _do_closeShift($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $success = true;
        $checkArray = array('Терминал ibox', 'Сверка итогов', 'Итоги совпали');
        $this->modifyStringArrayAsSberbank($checkArray);
        $result['Cheque'] = $checkArray;
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }
    
    /**
     * Здесь должен быть отчет о смене, но в ibox его нет
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:XReport&encode=TEXT
     */
    protected function _do_xReport($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $success = true;
        $checkArray = array('Терминал ibox', 'x-отчеты не поддерживаются');
        $this->modifyStringArrayAsSberbank($checkArray);
        $result['Cheque'] = $checkArray;
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }

    private function getPayCommand($login, $password, $amount, $model, $port, $singleStepEmv = 1, $description = "VirtualPospayment") {
        return sprintf('%s Monitor 1;Enable %s %s;SetCredentials %s %s;StartPayment %s %s \\"%s\\"', self::PATH_TO_EXE, $model, $port, $login, $password, $amount, $singleStepEmv, $description);
    }

    private function getCancelCommand($login, $password, $transactionId, $amount, $model, $port, $mode = "cancel") {
        if(!isset($transactionId)) {
            $transactionId = '\\" \\"';
        }
        return sprintf("%s Monitor 1;Enable %s %s;SetCredentials %s %s;StartReverse %s %s %s", self::PATH_TO_EXE, $model, $port, $login, $password, $mode, $transactionId, $amount);
    }

    private function executeCommand($command) {
        $result = shell_exec($command);
        return $this->fixEncoding($result);
    }

    private function getValue($data, $sender, $value) {
        $nameString = pinpad_ibox::IBOX_PREFIX . "_" . $sender . "_" . $value;
        $lines = preg_split("/\\r\\n|\\r|\\n/", $data);
        $result = array();
        foreach ($lines as $line) {
            if (strpos($line, $nameString) === 0) {//line start with target name
                $splitResult = explode(":", $line, 2);
                if (count($splitResult) > 1) {
                    $result[] = $splitResult[1];
                }
            }
        }
        return $result;
    }

    private function fixEncoding($consoleOutput) {
        return iconv('cp866', 'UTF-8', $consoleOutput);
    }

    private function splitStringToArray($string) {
        return explode("&#10", $string);
    }

    /**
     * Добавляет в массив строк служебные команды чтобы быть похожим на документ от сбербанка
     * 
     * @param type $slipArray массив строк документа
     */
    private function modifyStringArrayAsSberbank(&$slipArray) {
        $slipArray[] = " ";
        $slipArray[] = " ";
        $slipArray[] = " ";
        $slipArray[] = " ";
        $slipArray[] = chr(1);
    }

}
