<?php

require 'core/pinpad.php';
require 'pinpad_sbrf_native.php';

/* The goal of this class is translate standard interface methods to native driver calls */

class pinpad_sbrf extends Pinpad {

    protected function _initialize() {
        parent::_initialize();
        if (!isset($this->_nativeDevice)) {
            $this->_nativeDevice = new PinpadSbrfNative();
        }
    }

    protected function _internalOpen($dev = NULL) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = isset($dev) ? $dev : $this->get_NativeDevice();
        $this->log(__METHOD__ . " Connecting to device...\n", 3);

        // нет никаких проверок подключения, ибо нет таких функций. Пока будем работать по факту

        $this->_connected = true;
        $this->log(__METHOD__ . " ----- Connected OK\n", 3);
        return true;
    }

    protected function _internalClose($dev = NULL) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = isset($dev) ? $dev : $this->get_NativeDevice();
        $this->log(__METHOD__ . " Disconnecting from device...\n", 3);

        // нет никаких проверок отключения, ибо нет таких функций. Пока будем работать по факту

        $this->_connected = false;
        $this->log(__METHOD__ . " Disonnected OK\n", 3);
        return true;
    }

    protected function _internalClear($dev = NULL) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();
        try {
            $device->Clear();
        } catch (Exception $e) {
            //Skip error
            $this->log(__METHOD__ . " Warning. Clear method not supported (or just error occured) " . $e->getMessage() . " \n", 2);
        }
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return true;
    }

    private function convertCheque($srcCheque) {
        $conv = iconv('windows-1251', 'UTF-8', $srcCheque);

        $lines = preg_split('/\n|\r\n?/', $conv);

        return $lines;
    }

    /*     * ********************************************* */
    // конкретная реализация

    /**
     * Оплата покупки
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:Pay&encode=TEXT&p={"amount":100}
     * 
     * @param type $params
     */
    protected function _do_pay($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();

        if (!isset($params['amount'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_PARAM_ERROR,
                'error' => 'Во входящих параметрах не найдено [amount]',
            );
        }

        if (!is_int($params['amount'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_AMOUNT_MISTAKE,
                'error' => 'Параметр [amount] не валиден, он должен быть int > 0',
            );
        }

        $this->_internalClear(); // чистим терминал перед
        $device->SParam('Amount', $params['amount']);
        $device->SParam('Track2', '1'); //B6761960000695065=15125010000013003078F
        $device->SParam('CardType', '1');

        $result = array('apiResult' => $device->NFun(4000));

        // получаем все что может передать нам терминал
        if ($result['apiResult'] == 0) {
            // получение чека
            $result['Cheque'] = $this->convertCheque($device->GParamString('Cheque'));

            // получение остальной информации по методам 4xxx
            $result['AmountClear'] = $device->GParamString('AmountClear');
            $result['Amount'] = $device->GParamString('Amount');
            $result['CardName'] = $device->GParamString('CardName');
            $result['CardType'] = $device->GParamString('CardType');
            $result['TrxDate'] = $device->GParamString('TrxDate');
            $result['TrxTime'] = $device->GParamString('TrxTime');
            $result['TermNum'] = $device->GParamString('TermNum');
            $result['MerchNum'] = $device->GParamString('MerchNum');
            $result['AuthCode'] = $device->GParamString('AuthCode');
            $result['RRN'] = $device->GParamString('RRN');
            $result['Hash'] = $device->GParamString('Hash');
            $result['MerchantTSN'] = $device->GParamString('MerchantTSN');
            $result['MerchantBatchNum'] = $device->GParamString('MerchantBatchNum');
//			$result['ClientCard'] = $device->GParamString('ClientCard');
//			$result['ClientExpiryDate'] = $device->GParamString('ClientExpiryDate');
            $result['OwnCard'] = $device->GParamString('OwnCard');

            $success = true; // удачный вызов апи
        } else {
            // не удачный вызов апи
            $result['errorinfo'] = $this->resolveTerminalError($result['apiResult']);
            $result['errornum'] = $result['apiResult'];

            $success = false;
        }
        // в противном случае вернется ошибка

        $this->log(__METHOD__ . " ----- end OK\n", 3);

        return compact('success', 'result');
    }

    /**
     * Отмена операции
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:PayCansel&encode=TEXT&p={"amount":100}
     * 
     * @param type $params
     */
    protected function _do_payCansel($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();

        if (!isset($params['amount'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_PARAM_ERROR,
                'error' => 'Во входящих параметрах не найдено [amount]',
            );
        }

        if (!is_int($params['amount'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_AMOUNT_MISTAKE,
                'error' => 'Параметр [amount] не валиден, он должен быть int > 0',
            );
        }

        $this->_internalClear(); // чистим терминал перед
        if (isset($params['RRN']) && !empty($params['RRN'])) {
            $device->SParam('RRN', $params['RRN']);
        }
        $device->SParam('Amount', $params['amount']);
        $device->SParam('Track2', '1'); //B6761960000695065=15125010000013003078F
        $device->SParam('CardType', '1');

        $result = array('apiResult' => $device->NFun(4002));

        // получаем все что может передать нам терминал
        if ($result['apiResult'] == 0) {
            // получение чека
            $result['Cheque'] = $this->convertCheque($device->GParamString('Cheque'));

            // получение остальной информации по методам 4xxx
            $result['AmountClear'] = $device->GParamString('AmountClear');
            $result['Amount'] = $device->GParamString('Amount');
            $result['CardName'] = $device->GParamString('CardName');
            $result['CardType'] = $device->GParamString('CardType');
            $result['TrxDate'] = $device->GParamString('TrxDate');
            $result['TrxTime'] = $device->GParamString('TrxTime');
            $result['TermNum'] = $device->GParamString('TermNum');
            $result['MerchNum'] = $device->GParamString('MerchNum');
            $result['AuthCode'] = $device->GParamString('AuthCode');
            $result['RRN'] = $device->GParamString('RRN');
            $result['Hash'] = $device->GParamString('Hash');
            $result['MerchantTSN'] = $device->GParamString('MerchantTSN');
            $result['MerchantBatchNum'] = $device->GParamString('MerchantBatchNum');
//			$result['ClientCard'] = $device->GParamString('ClientCard');
//			$result['ClientExpiryDate'] = $device->GParamString('ClientExpiryDate');
            $result['OwnCard'] = $device->GParamString('OwnCard');

            $success = true; // удачный вызов апи
        } else {
            // не удачный вызов апи
            $result['errorinfo'] = $this->resolveTerminalError($result['apiResult']);
            $result['errornum'] = $result['apiResult'];

            $success = false;
        }

        $this->log("Результат $success. Данные API: " . json_encode($result), 3);
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }

    /**
     * Оплата с транзакцией
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:PayWithTransaction&encode=TEXT&p={"amount":100}
     * 
     * @param type $params
     * @return type
     */
    protected function _do_payWithTransaction($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();

        if (!isset($params['amount'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_PARAM_ERROR,
                'error' => 'Во входящих параметрах не найдено [amount]',
            );
        }

        if (!is_int($params['amount'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_AMOUNT_MISTAKE,
                'error' => 'Параметр [amount] не валиден, он должен быть int > 0',
            );
        }

        $this->_internalClear(); // чистим терминал перед
        $device->SParam('Amount', $params['amount']);
        $device->SParam('Track2', '1'); //B6761960000695065=15125010000013003078F
        $device->SParam('CardType', '1');

        $result = array('apiResult' => $device->NFun(4000));

        // получаем все что может передать нам терминал
        if ($result['apiResult'] == 0) {
            // получение чека
            $result['Cheque'] = $this->convertCheque($device->GParamString('Cheque'));

            // получение остальной информации по методам 4xxx
            $result['AmountClear'] = $device->GParamString('AmountClear');
            $result['Amount'] = $device->GParamString('Amount');
            $result['CardName'] = $device->GParamString('CardName');
            $result['CardType'] = $device->GParamString('CardType');
            $result['TrxDate'] = $device->GParamString('TrxDate');
            $result['TrxTime'] = $device->GParamString('TrxTime');
            $result['TermNum'] = $device->GParamString('TermNum');
            $result['MerchNum'] = $device->GParamString('MerchNum');
            $result['AuthCode'] = $device->GParamString('AuthCode');
            $result['RRN'] = $device->GParamString('RRN');
            $result['Hash'] = $device->GParamString('Hash');
            $result['MerchantTSN'] = $device->GParamString('MerchantTSN');
            $result['MerchantBatchNum'] = $device->GParamString('MerchantBatchNum');
//			$result['ClientCard'] = $device->GParamString('ClientCard');
//			$result['ClientExpiryDate'] = $device->GParamString('ClientExpiryDate');
            $result['OwnCard'] = $device->GParamString('OwnCard');

            // производим немедленный вызов 6003
            $resultTransactionStart = array('apiResult' => $device->NFun(6003));
            $result['resultTransactionStart'] = $resultTransactionStart;

            // производим немедленный вызов 6001
            //$resultTransactionCommit = array('apiResult' => $device->NFun(6001));
            //$result['resultTransactionCommit'] = $resultTransactionCommit;
            // после чего пинпад отключатся и возвращается 99 ошибка
            // удалось установить транзакцию в «неподтвержденное» состояние
            $success = $resultTransactionStart['apiResult'] == 0;
        } else {
            // не удачный вызов апи
            $result['errorinfo'] = $this->resolveTerminalError($result['apiResult']);
            $result['errornum'] = $result['apiResult'];

            $success = false;
        }
        // в противном случае вернется ошибка

        $this->log("Результат $success. Данные API: " . json_encode($result), 3);
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }

    /**
     * Установить транзакцию в «подтвержденное» состояние
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:PayTransactionCommit&encode=TEXT
     * 
     * @param type $params
     */
    protected function _do_payTransactionCommit($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();

        $device->SParam('Amount', $params['amount']);
        $device->SParam('Track2', '1'); //B6761960000695065=15125010000013003078F
        $device->SParam('CardType', '1');
        $result = array('apiResult' => $device->NFun(6001));

        $success = $result['apiResult'] == 0;
        if ($result['apiResult'] == 0) {
            // ничего другого получить нельзя
        } else {
            $result['errorinfo'] = $this->resolveTerminalError($result['apiResult']);
            $result['errornum'] = $result['apiResult'];
        }

        $this->_internalClear();

        $this->log("Результат $success. Данные API: " . json_encode($result), 3);
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }

    /**
     * Отменить транзакцию немедленно
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:PayTransactionCansel&encode=TEXT
     * 
     * @param type $params
     */
    protected function _do_payTransactionCansel($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        if (isset($params['RRN']) && !empty($params['RRN'])) return $this->_do_payCansel($params);
        $device = $this->get_NativeDevice();

        $result = array('apiResult' => $device->NFun(6004));

        $success = $result['apiResult'] == 0;
        if ($result['apiResult'] == 0) {
            // ничего другого получить нельзя
        } else {
            $result['errorinfo'] = $this->resolveTerminalError($result['apiResult']);
            $result['errornum'] = $result['apiResult'];
        }

        $this->_internalClear();

        $this->log("Результат $success. Данные API: " . json_encode($result), 3);
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }

    /**
     * Закрыть смену
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:CloseShift&encode=TEXT
     * 
     * @param type $params
     * @return type
     */
    protected function _do_closeShift($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();
        if ($errResult = $this->_isDeviceFailed(__METHOD__))
            return $errResult;

        $fnum = 6000;

        $this->_internalClear(); // чистим терминал перед

        $result = array('apiResult' => $device->NFun($fnum));
        $cheque = $device->GParamString('Cheque');
        $device->NFun($fnum);// Без повторной сверки итогов будет отображаться предыдущий х-отчёт, пока не произойдёт первая продажа через пинпад.
        if ($result['apiResult'] == 0) {
            $result['Cheque'] = $this->convertCheque($cheque);

            $success = true;
        } else {
            $result['errorinfo'] = $this->resolveTerminalError($result['apiResult']);
            $result['errornum'] = $result['apiResult'];

            $success = false;
        }

        $this->_internalClear(); // чистим терминал после

        $this->log("Результат $success. Данные API: " . json_encode($result), 3);
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }

    /**
     * Отчет по картам с магнитной полосов (не закрывает смену)
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:XReport&encode=TEXT
     * 
     * @param type $params
     * @return type
     */
    protected function _do_xReport($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();
        if ($errResult = $this->_isDeviceFailed(__METHOD__))
            return $errResult;

        $fnum = 6002;

        $this->_internalClear(); // чистим терминал перед

        $result = array('apiResult' => $device->NFun($fnum));
        if ($result['apiResult'] == 0) {
            $result['Cheque'] = $this->convertCheque($device->GParamString('Cheque'));

            $success = true;
        } else {
            $result['errorinfo'] = $this->resolveTerminalError($result['apiResult']);
            $result['errornum'] = $result['apiResult'];

            $success = false;
        }

        $this->_internalClear(); // чистим терминал после

        $this->log("Результат $success. Данные API: " . json_encode($result), 3);
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }

    public function GetInfo() {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();
        if ($errResult = $this->_isDeviceFailed(__METHOD__))
            return $errResult;

        $fnum = 5002;

        $this->_internalClear(); // чистим терминал перед

        $result = array('apiResult' => $device->NFun($fnum));
        if ($result['apiResult'] == 0) {
            $result['CardName'] = $device->GParamString('CardName');
            $result['CardType'] = $device->GParamString('CardType');
            $result['TrxDate'] = $device->GParamString('TrxDate');
            $result['TrxTime'] = $device->GParamString('TrxTime');
            $result['TermNum'] = $device->GParamString('TermNum');
//			$result['ClientCard'] = $device->GParamString('ClientCard');
//			$result['ClientExpiryDate'] = $device->GParamString('ClientExpiryDate');
            $result['Hash'] = $device->GParamString('Hash');
            $result['CardData'] = $device->GParamString('CardData');
            $result['OwnCard'] = $device->GParamString('OwnCard');

            $success = true;
        } else {
            $result['errorinfo'] = $this->resolveTerminalError($result['apiResult']);
            $result['errornum'] = $result['apiResult'];

            $success = false;
        }

        $this->_internalClear(); // чистим терминал после

        $this->log("Результат $success. Данные API: " . json_encode($result), 3);
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }

    public function GetBalance() {
        $this->log(__METHOD__ . " ----- start\n", 3);
        $device = $this->get_NativeDevice();
        if ($errResult = $this->_isDeviceFailed(__METHOD__))
            return $errResult;

        $fnum = 5000;

        $this->_internalClear(); // чистим терминал перед

        $result = array('apiResult' => $device->NFun($fnum));
        if ($result['apiResult'] == 0) {
            $result['CardName'] = $device->GParamString('CardName');
            $result['CardType'] = $device->GParamString('CardType');
            $result['TrxDate'] = $device->GParamString('TrxDate');
            $result['TrxTime'] = $device->GParamString('TrxTime');
            $result['TermNum'] = $device->GParamString('TermNum');
//			$result['ClientCard'] = $device->GParamString('ClientCard');
//			$result['ClientExpiryDate'] = $device->GParamString('ClientExpiryDate');
            $result['Hash'] = $device->GParamString('Hash');
            $result['CardData'] = $device->GParamString('CardData');
            $result['OwnCard'] = $device->GParamString('OwnCard');

            $success = true;
        } else {
            $result['errorinfo'] = $this->resolveTerminalError($result['apiResult']);
            $result['errornum'] = $result['apiResult'];

            $success = false;
        }

        $this->_internalClear(); // чистим терминал после

        $this->log("Результат $success. Данные API: " . json_encode($result), 3);
        $this->log(__METHOD__ . " ----- end OK\n", 3);
        return compact('success', 'result');
    }

    /**
     * Отменить транзакцию немедленно
     * @param type $params
     */
    public function Nfunc($params) {
        if (!isset($params['n'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_PARAM_ERROR,
                'error' => 'Во входящих параметрах не найдено [amount]',
            );
        }

        if (!is_int($params['n'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_AMOUNT_MISTAKE,
                'error' => 'Параметр [amount] не валиден, он должен быть int > 0',
            );
        }
        $device = $this->get_NativeDevice();

        $nfunc = $params['n'];

        $this->_internalClear();

        $result = array('apiResult' => $device->NFun($nfunc));
        $success = $result['apiResult'] == 0;

        //$Hash = $device->GParamString('Hash');
        //$CardData = $device->GParamString('CardData');
        $Cheque = $this->convertCheque($device->GParamString('Cheque'));

        return compact('success', 'result', 'Cheque');
    }

    /*     * ********************************************* */

    /**
     * Преобразование номера ошибки в тектовый вид. Согластно документу "Коды ошибок UPOS подробно.doc"
     * @param type $id
     * @return string
     */
    private function resolveTerminalError($id) {
        switch ($id) {
            case 12:
                return 'Ошибка возникает обычно в ДОС-версиях. Возможных причин две:
1.	В настройках указан неверный тип пинпада. Должно быть РС-2, а указано РС-3.
2.	Если ошибка возникает неустойчиво, то скорее всего виноват СОМ-порт. Он или нестандартный, или неисправный. Попробовать перенести пинпад на другой порт, а лучше – на USB';

            case 99:
                return 'Нарушился контакт с пинпадом, либо невозможно открыть указанный СОМ-порт (он или отсутствует в системе, или захвачен другой программой)';

            case 361:
            case 362:
            case 363:
            case 364:
                return 'Нарушился контакт с чипом карты. Чип не читается. Попробовать вставить другую карту. Если ошибка возникает на всех картах – неисправен чиповый ридер пинпада';

            case 403:
                return 'Клиент ошибся при вводе ПИНа (СБЕРКАРТ)';

            case 405:
                return 'ПИН клиента заблокирован  (СБЕРКАРТ)';

            case 444:
            case 507:
                return 'Истек срок действия карты  (СБЕРКАРТ)';

            case 518:
                return 'На терминале установлена неверная дата';

            case 521:
                return 'На карте недостаточно средств  (СБЕРКАРТ)';

            case 572:
                return 'Истек срок действия карты  (СБЕРКАРТ)';

            case 574:
            case 579:
                return 'Карта заблокирована  (СБЕРКАРТ)';

            case 584:
            case 585:
                return 'Истек период обслуживания карты  (СБЕРКАРТ)';

            case 705:
            case 706:
            case 707:
                return 'Карта заблокирована  (СБЕРКАРТ)';

            case 708:
            case 709:
                return 'ПИН клиента заблокирован  (СБЕРКАРТ)';

            case 2000:
                return 'Операция прервана нажатием клавиши ОТМЕНА. Другая возможная причина – не проведена предварительная сверка итогов, и на терминале еще нет сеансовых ключей';

            case 2002:
                return 'Клиент слишком долго вводит ПИН. Истек таймаут';

            case 2004:
            case 2005:
            case 2006:
            case 2007:
            case 2405:
            case 2406:
            case 2407:
                return 'Карта заблокирована (СБЕРКАРТ)';

            case 3001:
                return 'Недостаточно средств для загрузки на карту  (СБЕРКАРТ)';

            case 3002:
                return 'По карте клиента числится прерванная загрузка средств  (СБЕРКАРТ)';

            case 3019:
            case 3020:
            case 3021:
                return 'На сервере проводятся регламентные работы  (СБЕРКАРТ)';

            case 4100:
                return 'Нет связи с банком при удаленной загрузке. Возможно, на терминале неверно задан параметр «Код региона и участника для удаленной загрузки»';

            case 4101:
            case 4102:
                return 'Карта терминала не проинкассирована';

            case 4103:
            case 4104:
                return 'Ошибка обмена с чипом карты';

            case 4108:
                return 'Неправильно введен или прочитан номер карты (ошибка контрольного разряда)';

            case 4110:
            case 4111:
            case 4112:
                return 'Требуется проинкассировать карту терминала (СБЕРКАРТ)';

            case 4113:
            case 4114:
                return 'Превышен лимит, допустимый без связи с банком (СБЕРКАРТ)';

            case 4115:
                return 'Ручной ввод для таких карт запрещен';

            case 4116:
                return 'Введены неверные 4 последних цифры номера карты';

            case 4117:
                return 'Клиент отказался от ввода ПИНа';

            case 4119:
                return 'Нет связи с банком. 
Другая возможная причина – неверный ключ KLK для пинпада Verifone pp1000se или встроенного пинпада Verifone.
Если терминал Verifone работает по Ethernet, то иногда избавиться от ошибки можно, понизив скорость порта с 115200 до 57600 бод';

            case 4120:
                return 'В пинпаде нет ключа KLK';

            case 4121:
                return 'Ошибка файловой структуры терминала. Невозможно записать файл BTCH.D';

            case 4122:
                return 'Ошибка смены ключей: либо на хосте нет нужного KLK, либо в настройках терминала указан неверный мерчант';

            case 4123:
                return 'На терминале нет сеансовых ключей';

            case 4124:
                return 'На терминале нет мастер-ключей';

            case 4125:
                return 'На карте есть чип, а прочитана была магнитная полоса';

            case 4128:
                return 'Неверный МАС-код при сверке итогов. Вероятно, неверный ключ KLK';

            case 4130:
                return 'Память терминала заполнена. Пора делать сверку итогов (лучше несколько раз подряд, чтобы почистить старые отчеты)';

            case 4131:
                return 'Установлен тип пинпада РС-2, но с момента последней прогрузки параметров пинпад был заменен (изменился его серийный номер). Необходимо повторно пргрузить TLV-файл или выполнить удаленную загрузку';

            case 4132:
                return 'Операция отклонена картой. Возможно, карту вытащили из чипового ридера до завершения печати чека. Повторить операцию заново. Если ошибка возникает постоянно, возможно, карта неисправна';

            case 4134:
                return 'Слишком долго не выполнялась сверка итогов на терминале (прошло более 5 дней с момента последней операции)';

            case 4135:
                return 'Нет SAM-карты для выбранного отдела (СБЕРКАРТ)';

            case 4136:
                return 'Требуется более свежая версия прошивки в пинпаде';

            case 4137:
                return 'Ошибка при повторном вводе нового ПИНа';

            case 4138:
                return 'Номер карты получателя не может совпадать с номером карты отправителя';

            case 4139:
                return 'В настройках терминала нет ни одного варианта связи, пригодного для требуемой операции';

            case 4140:
                return 'Неверно указаны сумма или код авторизации в команде SUSPEND из кассовой программы';

            case 4141:
                return 'Невозможно выполнить команду SUSPEND: не найден файл SHCN.D';

            case 4142:
                return 'Не удалось выполнить команду ROLLBACK из кассовой прграммы';

            case 4143:
                return 'На терминале слишком старый стоп-лист';

            case 4144:
            case 4145:
            case 4146:
            case 4147:
                return 'Неверный формат стоп-листа на терминале (для торговли в самолете без авторизации)';

            case 4148:
                return 'Карта в стоп-листе';

            case 4149:
                return 'На карте нет фамилии держателя';

            case 4150:
                return 'Превышен лимит, допустимый без связи с банком (для торговли на борту самолета без авторизации)';

            case 4151:
                return 'Истек срок действия карты (для торговли на борту самолета без авторизации)';

            case 4152:
                return 'На карте нет списка транзакций (ПРО100)';

            case 4153:
                return 'Список транзакций на карте имеет неизвестный формат (ПРО100)';

            case 4154:
                return 'Невозможно распечатать список транзакций карты, потому что его можно считать только с чипа, а прочитана магнитная полоса (ПРО100)';

            case 4155:
                return 'Список транзакций пуст (ПРО100)';

            case 4160:
                return 'Неверный ответ от карты при считывании биометрических данных';

            case 4161:
                return 'На терминале нет файла с биометрическим сертификатом BSCP.CR';

            case 4162:
            case 4163:
            case 4164:
                return 'Ошибка расшифровки биометрического сертификата карты. Возможно, неверный файл BSCP.CR';

            case 4165:
            case 4166:
            case 4167:
                return 'Ошибка взаимной аутентификации биосканера и карты. Возможно, неверный файл BSCP.CR';

            case 4168:
            case 4169:
                return 'Ошибка расшифровки шаблонов пальцев, считанных с карты';

            case 4171:
                return 'В ответе хоста на запрос enrollment’a нет биометрической криптограммы';

            case 4202:
                return 'Сбой при удаленной загрузке: неверное смещение в данных';

            case 4203:
                return 'Не указанный или неверный код активации при удаленной загрузке';

            case 4208:
                return 'Ошибка удаленной загрузки: на сервере не активирован какой-либо шаблон для данного терминала';

            case 4209:
                return 'Ошибка удаленной загрузки: на сервере проблемы с доступом к БД';

            case 4211:
                return 'На терминале нет EMV-ключа с номером 62 (он нужен для удаленной загрузки)';

            case 4300:
                return 'Недостаточно параметров при запуске модуля sb_pilot. В командной строке указаны не все требуемые параметры';

            case 4301:
                return 'Кассовая программа передала в UPOS недопустимый тип операции';

            case 4302:
                return 'Кассовая программа передала в UPOS недопустимый тип карты';

            case 4303:
                return 'Тип карты, переданный из кассовой программы, не значится в настройках UPOS. Возможно, на диске кассы имеется несколько каталогов с библиотекой UPOS. Банковский инженер настраивал один экземпляр, а кассовая программа обращается к другому, где никаких настроек (а значит, и типов карт) нет';

            case 4305:
                return 'Ошибка инициализации библиотеки sb_kernel.dll. Кассовая программа ожидает библиотеку с более свежей версией';

            case 4306:
                return 'Библиотека sb_kernel.dll не была инициализирована. Эта ошибка может разово возникать после обновления библиотеки через удаленную загрузку. Нужно просто повторить операцию';

            case 4308:
                return 'В старых версиях этим кодом обозначалась любая из проблем, которые сейчас обозначаются кодами 4331-4342';

            case 4309:
                return 'Печатать нечего. Эта ошибка возникает в интегрированных решениях, которые выполнены не вполне корректно: в случае любой ошибки (нет связи, ПИН неверен, неверный ключ KLK и т.д.) кассовая программа все равно запрашивает у библиотеки sb_kernel.dll образ чека для печати. Поскольку по умолчанию библиотека при отказах чек не формирует, то на запрос чека она возвращает кассовой программе код 4309 – печатать нечего, нет документа для печати. Исходный код ошибки (тот, который обозначает причину отказа) кассовая программа при этом забывает';

            case 4310:
                return 'Кассовая программа передала в UPOS недопустимый трек2';

            case 4313:
                return 'В кассовой программе значится один номер карты, а через UPOS считан другой';

            case 4314:
                return 'Кассовая программа передала код операции «Оплата по международной карте», а вставлена была карта СБЕРКАРТ';

            case 4332:
                return 'Сверка итогов не выполнена (причина неизвестна, но печатать в итоге нечего)';

            case 4333:
                return 'Распечатать контрольную ленту невозможно (причина неизвестна, но печатать в итоге нечего)';

            case 4334:
                return 'Карта не считана. Либо цикл ожидания карты прерван нажатием клавиши ESC, либо просто истек таймаут.';

            case 4335:
                return 'Сумма не введена при операции ввода слипа';

            case 4336:
                return 'Из кассовой программы передан неверный код валюты';

            case 4337:
                return 'Из кассовой программы передан неверный тип карты';

            case 4338:
                return 'Вызвана операция по карте СБЕРКАРТ, но прочитать карту СБЕРКАРТ не удалось';

            case 4339:
                return 'Вызвана недопустимая операция по карте СБЕРКАРТ';

            case 4340:
                return 'Ошибка повторного считывания карты СБЕРКАРТ';

            case 4341:
                return 'Вызвана операция по карте СБЕРКАРТ, но вставлена карта другого типа, либо не вставлена никакая';

            case 4342:
                return 'Ошибка: невозможно запустить диалоговое окно UPOS (тред почему-то не создается)';

            case 5002:
                return 'Карта криво выпущена и поэтому дает сбой на терминалах, поддерживающих режим Offline Enciphered PIN';

            case 5026:
                return 'Ошибка проверки RSA-подписи. На терминале отсутствует (или некорректный) один из ключей из раздела «Ключи EMV»';

            case 5063:
                return 'На карте ПРО100 нет списка транзакций';

            case 5100:
            case 5101:
            case 5102:
            case 5103:
            case 5104:
            case 5105:
            case 5106:
            case 5107:
            case 5108:
                return 'Нарушены данные на чипе карты';

            case 5109:
                return 'Срок действия карты истек';

            case 5110:
                return 'Срок действия карты еще не начался';

            case 5111:
                return 'Для этой карты такая операция не разрешена';

            case 5116:
            case 5120:
                return 'Клиент отказался от ввода ПИНа';

            case 5133:
                return 'Операция отклонена картой';

            default:
                return 'Вызов метода не удался, произошла ошибка №' . $id;
        }
    }

}
