<?php

require 'core/pinpad.php';

/* This class use exe file instead of OLE */

class pinpad_vtb extends Pinpad {

    const PATH_TO_EXE = "C:\\Arcus2\\CommandLineTool\\bin\\CommandLineTool.exe";
    const PATH_OUTPUT = "C:\\Arcus2\\output.dat";
    const PATH_CHECK = "C:\\Arcus2\\cheq.out";

    /**
     * Список умений данного устройства
     * 
     */
    private $_actions = array(
        'pay' => array(),
        'payCansel' => array(),
    );

    /**
     * Оплата покупки
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:Pay&encode=TEXT&p={"amount":100}
     * 
     * @param type $params
     */
    protected function _do_pay($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);

        if (!isset($params['amount'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_PARAM_ERROR,
                'error' => 'Во входящих параметрах не найдено [amount]',
            );
        }

        if (!is_int($params['amount'])) {
            return array(
                'success' => false,
                'errcode' => parent::EC_AMOUNT_MISTAKE,
                'error' => 'Параметр [amount] повреждён, он должен быть целочисленным и больше нуля',
            );
        }

        $amount = $params['amount'];
        $command = $this->getPayCommand($amount);
        $this->executeCommand($command);

        $responseCode = $this->getLine(0, self::PATH_OUTPUT);
        if ($responseCode == 0) {
            $success = true;
            $chequeLines = $this->getLinesAsArray(self::PATH_CHECK);
            $this->modifyStringArrayAsSberbank($chequeLines);
            $result['Cheque'] = $chequeLines;
            $result['ClientCard'] = $this->getLine(1, self::PATH_OUTPUT);
            $result['TermNum'] = $this->getLine(2, self::PATH_OUTPUT);
            $result['AuthCode'] = $this->getLine(3, self::PATH_OUTPUT);
            $result['CardName'] = $this->getCardNameFromCheck($chequeLines);
            $result['AmountClear'] = $this->getLine(6, self::PATH_OUTPUT);
            $result['RRN'] = $this->getRRNFromCheck($chequeLines);
            $result['Hash'] = $this->getLine(8, self::PATH_OUTPUT);
        } else {
            $success = false;
            $result['errorinfo'] = $this->getLine(5, self::PATH_OUTPUT);
            $result['errornum'] = $responseCode;
        }

        $this->log(__METHOD__ . " ----- end OK\n", 3);

        return compact('success', 'result');
    }

    /**
     * Отмена операции. Слово cansel написано так и распространено по всему VirtualPos по прихоти Артура Кривцова (wartur)
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:PayCansel&encode=TEXT&p={"amount":100}
     * 
     * @param type $params
     */
    protected function _do_payCansel($params) {
        $amount = isset($params['amount']) ? $params['amount'] : null;
        $RRN = isset($params['RRN']) && !empty($params['RRN']) ? $params['RRN'] : null;

        $command = $this->getCancelCommand($amount, $RRN);

        $this->executeCommand($command);

        $responseCode = $this->getLine(0, self::PATH_OUTPUT);
        if ($responseCode == 0) {
            $success = true;
            $chequeLines = $this->getLinesAsArray(self::PATH_CHECK);
            $this->modifyStringArrayAsSberbank($chequeLines);
            $result['Cheque'] = $chequeLines;
            $result['ClientCard'] = $this->getLine(1, self::PATH_OUTPUT);
            $result['TermNum'] = $this->getLine(2, self::PATH_OUTPUT);
            $result['AuthCode'] = $this->getLine(3, self::PATH_OUTPUT);
            $result['CardName'] = $this->getCardNameFromCheck($chequeLines);
            $result['AmountClear'] = $this->getLine(6, self::PATH_OUTPUT);
            $result['RRN'] = $this->getRRNFromCheck($chequeLines);
            $result['Hash'] = $this->getLine(8, self::PATH_OUTPUT);
        } else {
            $success = false;
            $result['errorinfo'] = $this->getLine(5, self::PATH_OUTPUT);
            $result['errornum'] = $responseCode;
        }

        $this->log(__METHOD__ . " ----- end OK\n", 3);

        return compact('success', 'result');
    }

    /**
     * Делает тоже самое, что и _do_payCansel. Добавлено для совместимости с терминалом сбербанка
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:PayTransactionCansel&encode=TEXT
     * 
     * @param type $params
     */
    protected function _do_payTransactionCansel($params) {
        return $this->_do_payCansel($params);
    }

    /**
     * Здесь должно быть закрытие смены, но ibox его не поддерживает, поэтому всегда возвращает true
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:CloseShift&encode=TEXT
     * 
     * @param type $params
     * @return type
     */
    protected function _do_closeShift($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);

        $command = $this->getZReportCommand();
        $this->executeCommand($command);

        $responseCode = $this->getLine(0, self::PATH_OUTPUT);
        if ($responseCode == 0) {
            $success = true;
            $chequeLines = $this->getLinesAsArray(self::PATH_CHECK);
            $this->modifyStringArrayAsSberbank($chequeLines);
            $result['Cheque'] = $chequeLines;
        } else {
            $success = false;
            $result['errorinfo'] = $this->getLine(5, self::PATH_OUTPUT);
            $result['errornum'] = $responseCode;
        }

        $this->log(__METHOD__ . " ----- end OK\n", 3);

        return compact('success', 'result');
    }

    /**
     * Здесь должен быть отчет о смене, но в ibox его нет
     * http://local.virtualpos.hb/api.php?a=pinpad:sbrf:XReport&encode=TEXT
     */
    protected function _do_xReport($params) {
        $this->log(__METHOD__ . " ----- start\n", 3);

        $command = $this->getXReportCommand();
        $this->executeCommand($command);

        $responseCode = $this->getLine(0, self::PATH_OUTPUT);
        if ($responseCode == 0) {
            $success = true;
            $chequeLines = $this->getLinesAsArray(self::PATH_CHECK);
            $this->modifyStringArrayAsSberbank($chequeLines);
            $result['Cheque'] = $chequeLines;
        } else {
            $success = false;
            $result['errorinfo'] = $this->getLine(5, self::PATH_OUTPUT);
            $result['errornum'] = $responseCode;
        }

        $this->log(__METHOD__ . " ----- end OK\n", 3);

        return compact('success', 'result');
    }

    private function getPayCommand($amount) {
        return sprintf('%s /o1 /a%s /c643', self::PATH_TO_EXE, $amount);
    }

    private function getCancelCommand($amount, $rrn) {
        if (isset($amount) && (!isset($rrn) || empty($rrn))) {
            return sprintf('%s /o3 /a%s /c643', self::PATH_TO_EXE, $amount);
        } else if (isset($amount) && isset($rrn) && !empty($rrn)) {
            return sprintf('%s /o3 /a%s /c643 /r%s', self::PATH_TO_EXE, $amount, $rrn);
        } else {
            return sprintf('%s /o2', self::PATH_TO_EXE);
        }
    }

    private function getXReportCommand($isShort = true) {
        if ($isShort) {
            return sprintf('%s /o8', self::PATH_TO_EXE);
        } else {
            return sprintf('%s /o7', self::PATH_TO_EXE);
        }
    }

    private function getZReportCommand() {
        return sprintf('%s /o11', self::PATH_TO_EXE);
    }

    private function executeCommand($command) {
        return shell_exec($command);
    }

    private function fixEncoding($line) {
        return iconv('cp1251', 'UTF-8', $line);
    }

    /**
     * Метод ищет название карты в строках чека. В случае неудачи возвращается пустая строка.
     * @param type $linesArray
     * @return string
     */
    private function getCardNameFromCheck(&$linesArray) {
        foreach ($linesArray as $value) {
            preg_match('/Карта:(\w*) /', $value, $matches);
            if (isset($matches) && count($matches) === 2)
                return $matches[1];
        }
        return "";
    }
    
    /**
     * Метод ищет RRN транзакции в строках чека. В случае неудачи возвращается null.
     * @param type $linesArray
     * @return string
     */
    function getRRNFromCheck(&$linesArray) {
        foreach ($linesArray as $value) {
            preg_match('/.*RRN: *([0-9]*)/', $value, $matches);
            if (isset($matches) && count($matches) === 2)
                return $matches[1];
        }
        return null;
    }

    /**
     * Добавляет в массив строк служебные команды чтобы быть похожим на документ от сбербанка
     * 
     * @param type $slipArray массив строк документа
     */
    private function modifyStringArrayAsSberbank(&$slipArray) {
        $slipArray[] = " ";
        $slipArray[] = " ";
        $slipArray[] = " ";
        $slipArray[] = " ";
        $slipArray[] = chr(1);
    }

    private function getLine($lineNumber, $fileName) {
        $lines = file($fileName, FILE_IGNORE_NEW_LINES);
        if ($lines) {
            return $this->fixEncoding($lines[$lineNumber]);
        } else {
            return false;
        }
    }

    private function getLinesAsArray($fileName) {
        $lines = file($fileName, FILE_IGNORE_NEW_LINES);
        $fixedLines = array();
        if ($lines) {
            foreach ($lines as $key => $value) {
                $fixedLines[$key] = $this->fixEncoding($value);
            }
            return $fixedLines;
        } else {
            return false;
        }
    }

}
