<?php
/**
 * Пример вызова:
 * http://127.0.0.1:8080/api.php?a=scale:atol8:GetVersion
 * http://127.0.0.1:8080/api.php?a=scale:atol8:test&com=4:-1:38&loglevel=3
 */

require 'core/scale.php';
require 'scale_atol8_native.php';

class scale_atol8 extends Scale {

    const DEFAULT_BAUD_RATE = 7; // 9600

    protected $_baudValues = array(
        3 => '1200',
        4 => '2400',
        5 => '4800',
        7 => '9600',
        9 => '14400',
        10 => '19200'
    );

    protected $__LastError = 0;
	protected $__LastErrorMsg = '';

	public function __construct($loglevel = 0) {
		parent::__construct($loglevel);
	}

	protected function _getActionList(){
		$result = parent::_getActionList();
		$result['test'] = array();
		return $result;
	}

	protected function _initialize() {
		parent::_initialize();
		$this->__LastError = 0;
		$this->__LastErrorMsg = '';
		if (!isset($this->_nativeDevice)) {
			$this->_nativeDevice = new AtolScaleNative();
		}
	}

	protected function _internalOpen($dev = NULL) {
        $this->log(__METHOD__ . " ----- start\n", 3);
		$device = isset($dev) ? $dev : $this->get_NativeDevice();

		if ($device->DeviceCount <= 0) {
            $this->log(__METHOD__ . " ----- Try to add new logical device\n", 3);
            try {
                $device->AddDevice();
                $this->log(__METHOD__ . " ----- AddDevice function finished. Check results...\n", 3);
                if ($device->ResultCode !== 0) {
                    $this->__LastError = $device->ResultCode;
                    $this->__LastErrorMsg = $device->ResultDescription;
                    $this->log(__METHOD__ . " ----- ERROR AddDevice. {$device->ResultDescription}\n", 1);
                    return false;
                }
                $this->log(__METHOD__ . " ----- Device added\n", 3);
            } catch (Exception $e) {
                $this->__LastError = 1;
                $this->__LastErrorMsg = $e->getMessage();
                $this->log(__METHOD__ . " ----- ERROR AddDevice exception. {$e->getMessage()}\n", 1);
                return false;
            }
        }
				$this->log(__METHOD__ . " Connecting to device...\n", 3);
		if ($device->DeviceEnabled) $device->DeviceEnabled = false;
		if ($this->_deviceCOMPortNumber != -1) {
		    $device->PortNumber = $this->_deviceCOMPortNumber;
        }

		if ($this->_deviceCOMPortBaud != -1) {
			if (! array_key_exists($this->_deviceCOMPortBaud, $this->_baudValues) ) {
				$this->__LastError = 1;
				$this->__LastErrorMsg = "Неверно задана скорость для COM-порта (значение: {$this->_deviceCOMPortBaud})";
				$this->log(__METHOD__ . " ----- ERROR connecting. Bad com port baud value: {$this->_deviceCOMPortBaud}\n", 3);
				return false;
			}
			$device->BaudRate = $this->_baudValues[$this->_deviceCOMPortBaud];
		} else {
            $device->BaudRate = self::DEFAULT_BAUD_RATE;
        }
		//
        if ($this->_deviceCOMPortNumber >= 0) {
            $device->PortNumber = 1000 + $this->_deviceCOMPortNumber;
        }
		$device->BaudRate = 7;
		$device->Parity = 0;
		$device->Model = 38;
		$device->DecimalPoint = 3;
		//
        $device->Parity = 0;

		$this->log(__METHOD__ . " Using COM-port: {$device->PortNumber}, settings: {$device->DeviceSettings}...\n", 3);

		try {
			$device->DeviceEnabled = true;
		} catch (Exception $e) {
			$this->log(__METHOD__ . " ----- ERROR connecting. " . $e->getMessage() . " \n", 1);
			$this->__LastError = 1;
			$this->__LastErrorMsg = $e->getMessage();
			return false;
		}
		$this->_connected = true;
		$this->log(__METHOD__ . " ----- Connected OK\n", 3);
		return true;
	}

	protected function _internalClose($dev = NULL) {
		$this->log(__METHOD__ . " ----- start\n", 3);
		$device = isset($dev) ? $dev : $this->get_NativeDevice();
		$this->log(__METHOD__ . " Disconnecting from device...\n", 3);
		try {
			$device->DeviceEnabled = false;
		} catch (Exception $e) {
			$this->log(__METHOD__ . " ----- ERROR closing comm port. " . $e->getMessage() . " \n", 1);
			$this->__LastError = 1;
			$this->__LastErrorMsg = $e->getMessage();
			return false;
		}
		$this->_connected = false;
		$this->log(__METHOD__ . " Disconnected OK\n", 3);
		return true;
	}

	protected function _internalGetLastError($dev = NULL) {
		return $this->__LastError;
	}

	protected function _internalGetLastErrorMessage($dev = NULL) {
		return $this->__LastErrorMsg;
	}


	/*OVERRIDE DEVICE METHODS*/

	/**
	 * Устанавливает соединение с устройством для дальнейшей работы (открывает COM порт)
	 *
	 * @param
	 * @return bool
	 */
	public function Open() {
		parent::Open();
		$result = $this->_internalOpen();
		if (!$result) return $this->_isDeviceFailed(__METHOD__);
		return array('success' => true);
	}

	/**
	 * Закрывает соединение с устройством (закрывает COM порт)
	 *
	 * @param
	 * @return bool
	 */
	public function Close() {
		parent::Close();
		$result = $this->_internalClose();
		if (!$result) return $this->_isDeviceFailed(__METHOD__);
		return array('success' => true);
	}

	/**
	 * Возвращает ИСТИНА если соединение с устройством установлено
	 *
	 * @param
	 * @return bool
	 */
	public function IsOpen() {
		$result = $this->_internalIsOpen();
		return array('success' => true, 'isopen' => $result);
	}

    /**
     * Инициализирует весы
     *
     * @param array $params Массив параметров
     * @return bool
     */
    protected function _do_init($params){
        $this->__LastError = 0;
        $this->__LastErrorMsg = '';
        return array('success'=>true);
    }

    /**
     * Возвращает вес товара на весах
     *
     * @param $params
     * @return array
     */
    protected function _do_weight($params) {
        $this->__LastError = 0;
        $this->__LastErrorMsg = '';
		$device = $this->get_NativeDevice();
		$device->ReadWeight();
        return array('success'=>true, 'weight' => $device->Weight);
    }

	/**
	 * Метод выполняет перезапуск весов. При этом весы переходят в состояние аналогичное состоянию после их включения
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	protected function _do_reset($params){
		$this->log(__METHOD__ . " ----- start\n", 3);
		$device = $this->get_NativeDevice();
		try {
			$device->Reset(); // очистка
		} catch (Exception $e) {
			$this->__LastError = 1;
			$this->__LastErrorMsg = $e->getMessage();

			return $e->getMessage();
		}
		$this->log(__METHOD__ . " ----- end OK\n", 3);
		return array('success'=>true);
	}

	/**
	 * Выводит на дисплей строку
	 *
	 * @param array $params Массив параметров
	 * @return bool
	 */
	public function Test($params){
		$this->log(__METHOD__ . " ----- start\n", 3);
		try {
            $device = $this->get_NativeDevice();
            if ($this->_internalGetLastError($device) != 0)
                return array('success' => true, 'test' => false, 'error' => $this->_internalGetLastErrorMessage($device));
            $device->ReadWeight();
            return array('success'=>true, 'test' => $this->_internalGetLastError($device) === 0);
        } catch (Exception $e) {
            return array('success'=>true, 'test' => false, 'error' => $e->getMessage());
        }
	}


	/***************************************
	** PUBLIC METHODS
	*****************************************/

	/**
	 * Возвращает версию драйвера
	 *
	 * @param
	 * @return
	 */
	public function GetVersion() {
		$this->log(__METHOD__ . " ----- start\n", 3);
		$device = $this->get_NativeDevice();
		$version = $device->Version;
		$deviceDesc = iconv('windows-1251', 'utf-8', $device->DeviceDescription);
		return array('success'=>true, 'version' => $version, 'device' => $deviceDesc, 'time' => time());
	}
}
